@@ -27,13 +27,19 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: desc.c,v 1.12 2003/01/14 07:53:31 roland Exp $
+ *	$Id: desc.c,v 1.16 2004/10/20 02:18:50 roland Exp $
  */
 
 #include "defs.h"
 
 #include <fcntl.h>
 #include <sys/file.h>
+#ifdef LINUX
+#include <inttypes.h>
+#endif
+#ifdef HAVE_SYS_EPOLL_H
+#include <sys/epoll.h>
+#endif
 
 #if HAVE_LONG_LONG_OFF_T
 /*
@@ -44,7 +50,7 @@
 #endif
 
 
-static struct xlat fcntlcmds[] = {
+static const struct xlat fcntlcmds[] = {
 	{ F_DUPFD,	"F_DUPFD"	},
 	{ F_GETFD,	"F_GETFD"	},
 	{ F_SETFD,	"F_SETFD"	},
@@ -142,7 +148,7 @@ static struct xlat fcntlcmds[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat fdflags[] = {
+static const struct xlat fdflags[] = {
 #ifdef FD_CLOEXEC
 	{ FD_CLOEXEC,	"FD_CLOEXEC"	},
 #endif
@@ -151,7 +157,7 @@ static struct xlat fdflags[] = {
 
 #ifdef LOCK_SH
 
-static struct xlat flockcmds[] = {
+static const struct xlat flockcmds[] = {
 	{ LOCK_SH,	"LOCK_SH"	},
 	{ LOCK_EX,	"LOCK_EX"	},
 	{ LOCK_NB,	"LOCK_NB"	},
@@ -161,7 +167,7 @@ static struct xlat flockcmds[] = {
 
 #endif /* LOCK_SH */
 
-static struct xlat lockfcmds[] = {
+static const struct xlat lockfcmds[] = {
 	{ F_RDLCK,	"F_RDLCK"	},
 	{ F_WRLCK,	"F_WRLCK"	},
 	{ F_UNLCK,	"F_UNLCK"	},
@@ -174,7 +180,7 @@ static struct xlat lockfcmds[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat whence[] = {
+static const struct xlat whence[] = {
 	{ SEEK_SET,	"SEEK_SET"	},
 	{ SEEK_CUR,	"SEEK_CUR"	},
 	{ SEEK_END,	"SEEK_END"	},
@@ -235,7 +241,7 @@ int getlk;
 
 static char *
 sprintflags(xlat, flags)
-struct xlat *xlat;
+const struct xlat *xlat;
 int flags;
 {
 	static char outstr[1024];
@@ -261,7 +267,7 @@ int
 sys_fcntl(tcp)
 struct tcb *tcp;
 {
-	extern struct xlat openmodes[];
+	extern const struct xlat openmodes[];
 
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
@@ -549,6 +555,138 @@ struct tcb *tcp;
 }
 #endif
 
+static struct xlat epollctls[] = {
+#ifdef EPOLL_CTL_ADD
+	{ EPOLL_CTL_ADD,	"EPOLL_CTL_ADD"	},
+#endif
+#ifdef EPOLL_CTL_MOD
+	{ EPOLL_CTL_MOD,	"EPOLL_CTL_MOD"	},
+#endif
+#ifdef EPOLL_CTL_DEL
+	{ EPOLL_CTL_DEL,	"EPOLL_CTL_DEL"	},
+#endif
+	{ 0,			NULL		}
+};
+
+static struct xlat epollevents[] = {
+#ifdef EPOLLIN
+	{ EPOLLIN,	"EPOLLIN"	},
+#endif
+#ifdef EPOLLPRI
+	{ EPOLLPRI,	"EPOLLPRI"	},
+#endif
+#ifdef EPOLLOUT
+	{ EPOLLOUT,	"EPOLLOUT"	},
+#endif
+#ifdef EPOLLRDNORM
+	{ EPOLLRDNORM,	"EPOLLRDNORM"	},
+#endif
+#ifdef EPOLLRDBAND
+	{ EPOLLRDBAND,	"EPOLLRDBAND"	},
+#endif
+#ifdef EPOLLWRNORM
+	{ EPOLLWRNORM,	"EPOLLWRNORM"	},
+#endif
+#ifdef EPOLLWRBAND
+	{ EPOLLWRBAND,	"EPOLLWRBAND"	},
+#endif
+#ifdef EPOLLMSG
+	{ EPOLLMSG,	"EPOLLMSG"	},
+#endif
+#ifdef EPOLLERR
+	{ EPOLLERR,	"EPOLLERR"	},
+#endif
+#ifdef EPOLLHUP
+	{ EPOLLHUP,	"EPOLLHUP"	},
+#endif
+#ifdef EPOLLONESHOT
+	{ EPOLLONESHOT,	"EPOLLONESHOT"	},
+#endif
+#ifdef EPOLLET
+	{ EPOLLET,	"EPOLLET"	},
+#endif
+	{ 0,		NULL		}
+};
+
+int
+sys_epoll_create(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp))
+		tprintf("%ld", tcp->u_arg[0]);
+	return 0;
+}
+
+#ifdef HAVE_SYS_EPOLL_H
+static void
+print_epoll_event(ev)
+struct epoll_event *ev;
+{
+	tprintf("{");
+	if (printflags(epollevents, ev->events) == 0)
+		tprintf("0");
+	/* We cannot know what format the program uses, so print u32 and u64
+	   which will cover every value.  */
+	tprintf(", {u32=%" PRIu32 ", u64=%" PRIu64 "}}",
+		ev->data.u32, ev->data.u64);
+}
+#endif
+
+int
+sys_epoll_ctl(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+                printxval(epollctls, tcp->u_arg[1], "EPOLL_CTL_???");
+		tprintf(", %ld, ", tcp->u_arg[2]);
+		if (tcp->u_arg[3] == 0)
+			tprintf("NULL");
+		else {
+#ifdef HAVE_SYS_EPOLL_H
+			struct epoll_event ev;
+			if (umove(tcp, tcp->u_arg[3], &ev) == 0)
+				print_epoll_event(&ev);
+			else
+#endif
+				tprintf("{...}");
+		}
+	}
+	return 0;
+}
+
+int
+sys_epoll_wait(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp))
+		tprintf("%ld, ", tcp->u_arg[0]);
+	else {
+		if (syserror(tcp))
+			tprintf("%lx", tcp->u_arg[1]);
+		else if (tcp->u_rval == 0)
+			tprintf("{}");
+		else {
+#ifdef HAVE_SYS_EPOLL_H
+			struct epoll_event evs[tcp->u_rval];
+			if (umove(tcp, tcp->u_arg[1], evs) == 0) {
+				unsigned long i;
+				tprintf("{");
+				for (i = 0; i < tcp->u_rval; ++i) {
+					if (i > 0)
+						tprintf(", ");
+					print_epoll_event(&evs[i]);
+				}
+				tprintf("}");
+			}
+			else
+#endif
+				tprintf("{...}");
+		}
+		tprintf(", %ld, %ld", tcp->u_arg[2], tcp->u_arg[3]);
+	}
+	return 0;
+}
 #endif /* LINUX */
 
 int
