@@ -28,7 +28,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: file.c,v 1.58 2004/07/12 07:44:08 roland Exp $
+ *	$Id: file.c,v 1.65 2004/10/20 02:24:19 roland Exp $
  */
 
 #include "defs.h"
@@ -205,7 +205,7 @@ struct stat_sparc64 {
  * Unix says r/w/rw are 0/1/2, so we make them true flags 1/2/3 by
  * adding 1.  Just remember to add 1 to any arg decoded with openmodes.
  */
-struct xlat openmodes[] = {
+const struct xlat openmodes[] = {
 	{ O_RDWR+1,	"O_RDWR"	},
 	{ O_RDONLY+1,	"O_RDONLY"	},
 	{ O_WRONLY+1,	"O_WRONLY"	},
@@ -312,7 +312,7 @@ struct tcb *tcp;
 }
 
 #ifdef LINUXSPARC
-struct xlat openmodessol[] = {
+const struct xlat openmodessol[] = {
 	{ 0,		"O_RDWR"	},
 	{ 1,		"O_RDONLY"	},
 	{ 2,		"O_WRONLY"	},
@@ -360,7 +360,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat access_flags[] = {
+static const struct xlat access_flags[] = {
 	{ F_OK,		"F_OK",		},
 	{ R_OK,		"R_OK"		},
 	{ W_OK,		"W_OK"		},
@@ -396,7 +396,7 @@ struct tcb *tcp;
 	return RVAL_OCTAL;
 }
 
-static struct xlat whence[] = {
+static const struct xlat whence[] = {
 	{ SEEK_SET,	"SEEK_SET"	},
 	{ SEEK_CUR,	"SEEK_CUR"	},
 	{ SEEK_END,	"SEEK_END"	},
@@ -431,6 +431,11 @@ sys_llseek (tcp)
 struct tcb *tcp;
 {
     if (entering(tcp)) {
+	/*
+	 * This one call takes explicitly two 32-bit arguments hi, lo,
+	 * rather than one 64-bit argument for which LONG_LONG works
+	 * appropriate for the native byte order.
+	 */
 	if (tcp->u_arg[4] == SEEK_SET)
 	    tprintf("%ld, %llu, ", tcp->u_arg[0],
 		    (((long long int) tcp->u_arg[1]) << 32
@@ -460,9 +465,7 @@ struct tcb *tcp;
 # if defined IA64 || defined X86_64 || defined ALPHA
 		(long long int) tcp->u_arg[1], tcp->u_arg[2]
 # else
-		(((long long int) tcp->u_arg[1]) << 32
-		 | ((unsigned long *) tcp->u_arg)[2]),
-		tcp->u_arg[3]
+		LONG_LONG(tcp->u_arg[1], tcp->u_arg[2]), tcp->u_arg[3]
 # endif
 		);
     }
@@ -544,7 +547,7 @@ struct tcb *tcp;
 
 /* several stats */
 
-static struct xlat modetypes[] = {
+static const struct xlat modetypes[] = {
 	{ S_IFREG,	"S_IFREG"	},
 	{ S_IFSOCK,	"S_IFSOCK"	},
 	{ S_IFIFO,	"S_IFIFO"	},
@@ -555,12 +558,12 @@ static struct xlat modetypes[] = {
 	{ 0,		NULL		},
 };
 
-static char *
+static const char *
 sprintmode(mode)
 int mode;
 {
 	static char buf[64];
-	char *s;
+	const char *s;
 
 	if ((mode & S_IFMT) == 0)
 		s = "";
@@ -737,7 +740,7 @@ long addr;
 #endif /* SPARC64 */
 #endif /* LINUXSPARC */
 
-struct xlat fileflags[] = {
+const struct xlat fileflags[] = {
 #ifdef FREEBSD
 	{ UF_NODUMP,	"UF_NODUMP"	},
 	{ UF_IMMUTABLE,	"UF_IMMUTABLE"	},
@@ -1292,7 +1295,7 @@ struct tcb *tcp;
 
 #include <sys/acl.h>
 
-struct xlat aclcmds[] = {
+const struct xlat aclcmds[] = {
 #ifdef SETACL
 	{ SETACL,	"SETACL"	},
 #endif
@@ -1357,7 +1360,7 @@ struct tcb *tcp;
 }
 
 
-struct xlat aclipc[] = {
+const struct xlat aclipc[] = {
 #ifdef IPC_SHM
 	{ IPC_SHM,	"IPC_SHM"	},
 #endif
@@ -1400,7 +1403,7 @@ struct tcb *tcp;
 
 #ifdef LINUX
 
-static struct xlat fsmagic[] = {
+static const struct xlat fsmagic[] = {
 	{ 0x73757245,	"CODA_SUPER_MAGIC"	},
 	{ 0x012ff7b7,	"COH_SUPER_MAGIC"	},
 	{ 0x1373,	"DEVFS_SUPER_MAGIC"	},
@@ -1437,13 +1440,13 @@ static struct xlat fsmagic[] = {
 
 #ifndef SVR4
 
-static char *
+static const char *
 sprintfstype(magic)
 int magic;
 {
 	static char buf[32];
 #ifdef LINUX
-	char *s;
+	const char *s;
 
 	s = xlookup(fsmagic, magic);
 	if (s) {
@@ -1994,7 +1997,7 @@ struct tcb *tcp;
 #endif /* LINUX */
 
 #if defined FREEBSD || defined LINUX
-struct xlat direnttypes[] = {
+const struct xlat direnttypes[] = {
 	{ DT_UNKNOWN,	"DT_UNKNOWN" 	},
 	{ DT_FIFO,	"DT_FIFO" 	},
 	{ DT_CHR,	"DT_CHR" 	},
@@ -2327,7 +2330,7 @@ struct tcb *tcp;
 
 #endif /* HAVE_SYS_ASYNCH_H */
 
-struct xlat xattrflags[] = {
+const struct xlat xattrflags[] = {
 #ifdef XATTR_CREATE
 	{ XATTR_CREATE,	 "XATTR_CREATE" },
 	{ XATTR_REPLACE, "XATTR_REPLACE" },
@@ -2335,6 +2338,37 @@ struct xlat xattrflags[] = {
 	{ 0,		 NULL }
 };
 
+static void
+print_xattr_val(tcp, failed, arg, insize, size)
+struct tcb *tcp;
+int failed;
+unsigned long arg;
+size_t size;
+{
+    unsigned char buf[4 * size + 1];
+    if (!failed && umoven(tcp, arg, size, &buf[3 * size]) >= 0) {
+	unsigned char *out = buf;
+	unsigned char *in = &buf[3 * size];
+	size_t i;
+	for (i = 0; i < size; ++i)
+	    if (isprint(in[i]))
+		*out++ = in[i];
+	    else {
+#define tohex(n) "0123456789abcdef"[n]
+		*out++ = '\\';
+		*out++ = 'x';
+		*out++ = tohex(in[i] / 16);
+		*out++ = tohex(in[i] % 16);
+	    }
+	/* Don't print terminating NUL if there is one.  */
+	if (in[i - 1] == '\0')
+	    out -= 4;
+	*out = '\0';
+	tprintf(", \"%s\", %zd", buf, insize);
+    } else
+	tprintf(", 0x%lx, %zd", arg, insize);
+}
+
 int
 sys_setxattr(tcp)
 struct tcb *tcp;
@@ -2343,8 +2377,8 @@ struct tcb *tcp;
 	printpath(tcp, tcp->u_arg[0]);
 	tprintf(", ");
 	printstr(tcp, tcp->u_arg[1], -1);
-	/* XXX Print value in format */
-	tprintf(", %p, %ld, ", (void *) tcp->u_arg[2], tcp->u_arg[3]);
+	print_xattr_val(tcp, 0, tcp->u_arg[2], tcp->u_arg[3], tcp->u_arg[3]);
+	tprintf(", ");
 	printflags(xattrflags, tcp->u_arg[4]);
     }
     return 0;
@@ -2357,8 +2391,8 @@ struct tcb *tcp;
     if (entering(tcp)) {
 	tprintf("%ld, ", tcp->u_arg[0]);
 	printstr(tcp, tcp->u_arg[1], -1);
-	/* XXX Print value in format */
-	tprintf(", %p, %ld, ", (void *) tcp->u_arg[2], tcp->u_arg[3]);
+	print_xattr_val(tcp, 0, tcp->u_arg[2], tcp->u_arg[3], tcp->u_arg[3]);
+	tprintf(", ");
 	printflags(xattrflags, tcp->u_arg[4]);
     }
     return 0;
@@ -2373,8 +2407,8 @@ struct tcb *tcp;
 	tprintf(", ");
 	printstr(tcp, tcp->u_arg[1], -1);
     } else {
-	/* XXX Print value in format */
-	tprintf(", %p, %ld", (void *) tcp->u_arg[2], tcp->u_arg[3]);
+	print_xattr_val(tcp, syserror(tcp), tcp->u_arg[2], tcp->u_arg[3],
+			tcp->u_rval);
     }
     return 0;
 }
@@ -2387,8 +2421,8 @@ struct tcb *tcp;
 	tprintf("%ld, ", tcp->u_arg[0]);
 	printstr(tcp, tcp->u_arg[1], -1);
     } else {
-	/* XXX Print value in format */
-	tprintf(", %p, %ld", (void *) tcp->u_arg[2], tcp->u_arg[3]);
+	print_xattr_val(tcp, syserror(tcp), tcp->u_arg[2], tcp->u_arg[3],
+			tcp->u_rval);
     }
     return 0;
 }
@@ -2441,3 +2475,55 @@ struct tcb *tcp;
     }
     return 0;
 }
+
+
+static const struct xlat advise[] = {
+  { POSIX_FADV_NORMAL,		"POSIX_FADV_NORMAL"	},
+  { POSIX_FADV_RANDOM,		"POSIX_FADV_RANDOM"	},
+  { POSIX_FADV_SEQUENTIAL,	"POSIX_FADV_SEQUENTIAL"	},
+  { POSIX_FADV_WILLNEED,	"POSIX_FADV_WILLNEED"	},
+  { POSIX_FADV_DONTNEED,	"POSIX_FADV_DONTNEED"	},
+  { POSIX_FADV_NOREUSE,		"POSIX_FADV_NOREUSE"	},
+  { 0,				NULL			}
+};
+
+
+#ifdef LINUX
+int
+sys_fadvise64(tcp)
+struct tcb *tcp;
+{
+    if (entering(tcp)) {
+	tprintf("%ld, %lld, %ld, ",
+		tcp->u_arg[0],
+# if defined IA64 || defined X86_64 || defined ALPHA
+		(long long int) tcp->u_arg[1], tcp->u_arg[2]);
+	printxval(advise, tcp->u_arg[3], "POSIX_FADV_???");
+#else
+		LONG_LONG(tcp->u_arg[1], tcp->u_arg[2]), tcp->u_arg[3]);
+	printxval(advise, tcp->u_arg[4], "POSIX_FADV_???");
+#endif
+    }
+    return 0;
+}
+#endif
+
+
+int
+sys_fadvise64_64(tcp)
+struct tcb *tcp;
+{
+    if (entering(tcp)) {
+	tprintf("%ld, %lld, %lld, ",
+		tcp->u_arg[0],
+# if defined IA64 || defined X86_64 || defined ALPHA
+		(long long int) tcp->u_arg[1], (long long int) tcp->u_arg[2]);
+	printxval(advise, tcp->u_arg[3], "POSIX_FADV_???");
+#else
+		LONG_LONG(tcp->u_arg[1], tcp->u_arg[2]),
+		LONG_LONG(tcp->u_arg[3], tcp->u_arg[4]));
+	printxval(advise, tcp->u_arg[5], "POSIX_FADV_???");
+#endif
+    }
+    return 0;
+}
