@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: io.c,v 1.10 2001/07/10 13:48:44 hughesj Exp $
+ *	$Id: io.c,v 1.19 2004/10/07 19:09:16 roland Exp $
  */
 
 #include "defs.h"
@@ -89,7 +89,7 @@ long addr;
 		tprintf("[]");
 		return;
 	}
-	  
+
 	if ((iov = (struct iovec *) malloc(len * sizeof *iov)) == NULL) {
 		fprintf(stderr, "No memory");
 		return;
@@ -161,8 +161,7 @@ struct tcb *tcp;
 		tprintf(", %lu, %ld", tcp->u_arg[2], tcp->u_arg[3]);
 #else
 		tprintf(", %lu, %llu", tcp->u_arg[2],
-				(((unsigned long long) tcp->u_arg[4]) << 32
-				 | tcp->u_arg[3]));
+			LONG_LONG(tcp->u_arg[3], tcp->u_arg[4]));
 #endif
 	}
 	return 0;
@@ -180,8 +179,7 @@ struct tcb *tcp;
 		tprintf(", %lu, %ld", tcp->u_arg[2], tcp->u_arg[3]);
 #else
 		tprintf(", %lu, %llu", tcp->u_arg[2],
-				(((unsigned long long) tcp->u_arg[4]) << 32
-				 | tcp->u_arg[3]));
+			LONG_LONG(tcp->u_arg[3], tcp->u_arg[4]));
 #endif
 	}
 	return 0;
@@ -198,8 +196,8 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		tprintf("%ld, %ld, %llu, %lu", tcp->u_arg[0], tcp->u_arg[1],
-			(((unsigned long long) tcp->u_arg[3]) << 32 |
-			 tcp->u_arg[2]), tcp->u_arg[4]);
+			LONG_LONG(tcp->u_arg[2], tcp->u_arg[3]),
+			tcp->u_arg[4]);
 	} else {
 		off_t offset;
 
@@ -231,6 +229,18 @@ struct tcb *tcp;
 #endif /* FREEBSD */
 
 #ifdef LINUX
+
+/* The SH4 ABI does allow long longs in odd-numbered registers, but
+   does not allow them to be split between registers and memory - and
+   there are only four argument registers for normal functions.  As a
+   result pread takes an extra padding argument before the offset.  This
+   was changed late in the 2.4 series (around 2.4.20).  */
+#if defined(SH)
+#define PREAD_OFFSET_ARG 4
+#else
+#define PREAD_OFFSET_ARG 3
+#endif
+
 int
 sys_pread(tcp)
 struct tcb *tcp;
@@ -242,8 +252,9 @@ struct tcb *tcp;
 			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printstr(tcp, tcp->u_arg[1], tcp->u_rval);
+		ALIGN64 (tcp, PREAD_OFFSET_ARG); /* PowerPC alignment restriction */
 		tprintf(", %lu, %llu", tcp->u_arg[2],
-			*(unsigned long long *)&tcp->u_arg[3]);
+			*(unsigned long long *)&tcp->u_arg[PREAD_OFFSET_ARG]);
 	}
 	return 0;
 }
@@ -255,8 +266,9 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
 		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
+		ALIGN64 (tcp, PREAD_OFFSET_ARG); /* PowerPC alignment restriction */
 		tprintf(", %lu, %llu", tcp->u_arg[2],
-			*(unsigned long long *)&tcp->u_arg[3]);
+			*(unsigned long long *)&tcp->u_arg[PREAD_OFFSET_ARG]);
 	}
 	return 0;
 }
@@ -280,6 +292,25 @@ struct tcb *tcp;
 	return 0;
 }
 
+int
+sys_sendfile64(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		loff_t offset;
+
+		tprintf("%ld, %ld, ", tcp->u_arg[0], tcp->u_arg[1]);
+		if (!tcp->u_arg[2])
+			tprintf("NULL");
+		else if (umove(tcp, tcp->u_arg[2], &offset) < 0)
+			tprintf("%#lx", tcp->u_arg[2]);
+		else
+			tprintf("[%llu]", (unsigned long long int) offset);
+		tprintf(", %lu", tcp->u_arg[3]);
+	}
+	return 0;
+}
+
 #endif /* LINUX */
 
 #if _LFS64_LARGEFILE || HAVE_LONG_LONG_OFF_T
@@ -315,25 +346,30 @@ struct tcb *tcp;
 	return 0;
 }
 #endif
- 
+
 int
 sys_ioctl(tcp)
 struct tcb *tcp;
 {
-	char *symbol;
+	const struct ioctlent *iop;
 
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
-		symbol = ioctl_lookup(tcp->u_arg[1]);
-		if (symbol)
-			tprintf("%s", symbol);
-		else
+		iop = ioctl_lookup(tcp->u_arg[1]);
+		if (iop) {
+			tprintf("%s", iop->symbol);
+			while ((iop = ioctl_next_match(iop)))
+				tprintf(" or %s", iop->symbol);
+		} else
 			tprintf("%#lx", tcp->u_arg[1]);
 		ioctl_decode(tcp, tcp->u_arg[1], tcp->u_arg[2]);
 	}
 	else {
-		if (ioctl_decode(tcp, tcp->u_arg[1], tcp->u_arg[2]) == 0)
+		int ret;
+		if (!(ret = ioctl_decode(tcp, tcp->u_arg[1], tcp->u_arg[2])))
 			tprintf(", %#lx", tcp->u_arg[2]);
+		else
+			return ret - 1;
 	}
 	return 0;
 }
