@@ -27,13 +27,18 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: ipc.c,v 1.3 2000/09/01 21:03:06 wichert Exp $
+ *	$Id: ipc.c,v 1.8 2004/09/04 03:39:20 roland Exp $
  */
 
 #include "defs.h"
 
 #if defined(LINUX) || defined(SUNOS4) || defined(FREEBSD)
 
+# ifdef HAVE_MQUEUE_H
+#  include <mqueue.h>
+# endif
+
+#include <fcntl.h>
 #include <sys/ipc.h>
 #include <sys/sem.h>
 #include <sys/msg.h>
@@ -58,7 +63,14 @@
 #define SEM_INFO 19
 #endif
 
-static struct xlat msgctl_flags[] = {
+#if defined LINUX && !defined IPC_64
+# define IPC_64 0x100
+#endif
+
+extern const struct xlat openmodes[];
+extern void printsigevent(struct tcb *tcp, long arg);
+
+static const struct xlat msgctl_flags[] = {
 	{ IPC_RMID,	"IPC_RMID"	},
 	{ IPC_SET,	"IPC_SET"	},
 	{ IPC_STAT,	"IPC_STAT"	},
@@ -70,7 +82,7 @@ static struct xlat msgctl_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat semctl_flags[] = {
+static const struct xlat semctl_flags[] = {
 	{ IPC_RMID,	"IPC_RMID"	},
 	{ IPC_SET,	"IPC_SET"	},
 	{ IPC_STAT,	"IPC_STAT"	},
@@ -89,7 +101,7 @@ static struct xlat semctl_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat shmctl_flags[] = {
+static const struct xlat shmctl_flags[] = {
 	{ IPC_RMID,	"IPC_RMID"	},
 	{ IPC_SET,	"IPC_SET"	},
 	{ IPC_STAT,	"IPC_STAT"	},
@@ -98,23 +110,26 @@ static struct xlat shmctl_flags[] = {
 	{ SHM_STAT,	"SHM_STAT"	},
 	{ SHM_INFO,	"SHM_INFO"	},
 #endif /* LINUX */
-#ifdef SHM_LOCK	
+#ifdef SHM_LOCK
 	{ SHM_LOCK,	"SHM_LOCK"	},
 #endif
-#ifdef SHM_UNLOCK	
+#ifdef SHM_UNLOCK
 	{ SHM_UNLOCK,	"SHM_UNLOCK"	},
-#endif	
+#endif
 	{ 0,		NULL		},
 };
 
-static struct xlat resource_flags[] = {
+static const struct xlat resource_flags[] = {
 	{ IPC_CREAT,	"IPC_CREAT"	},
 	{ IPC_EXCL,	"IPC_EXCL"	},
 	{ IPC_NOWAIT,	"IPC_NOWAIT"	},
+#ifdef SHM_HUGETLB
+	{ SHM_HUGETLB,	"SHM_HUGETLB"	},
+#endif
 	{ 0,		NULL		},
 };
 
-static struct xlat shm_flags[] = {
+static const struct xlat shm_flags[] = {
 #ifdef LINUX
 	{ SHM_REMAP,	"SHM_REMAP"	},
 #endif /* LINUX */
@@ -123,7 +138,7 @@ static struct xlat shm_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat msg_flags[] = {
+static const struct xlat msg_flags[] = {
 	{ MSG_NOERROR,	"MSG_NOERROR"	},
 #ifdef LINUX
 	{ MSG_EXCEPT,	"MSG_EXCEPT"	},
@@ -141,21 +156,27 @@ struct tcb *tcp;
 		else
 			tprintf("IPC_PRIVATE");
 		tprintf(", ");
-		if (printflags(resource_flags, tcp->u_arg[1]) != 0)
+		if (printflags(resource_flags, tcp->u_arg[1] & ~0777) != 0)
 			tprintf("|");
-		tprintf("%#lo", tcp->u_arg[1] & 0666);
+		tprintf("%#lo", tcp->u_arg[1] & 0777);
 	}
 	return 0;
 }
 
+#ifdef IPC_64
+# define PRINTCTL(flagset, arg, dflt) \
+	if ((arg) & IPC_64) tprintf("IPC_64|"); \
+	printxval((flagset), (arg) &~ IPC_64, dflt)
+#else
+# define PRINTCTL printxval
+#endif
+
 int sys_msgctl(tcp)
 struct tcb *tcp;
 {
-	char *cmd = xlookup(msgctl_flags, tcp->u_arg[1]);
-
 	if (entering(tcp)) {
-		tprintf("%lu", tcp->u_arg[0]);
-		tprintf(", %s", cmd == NULL ? "MSG_???" : cmd);
+		tprintf("%lu, ", tcp->u_arg[0]);
+		PRINTCTL(msgctl_flags, tcp->u_arg[1], "MSG_???");
 #ifdef LINUX
 		tprintf(", %#lx", tcp->u_arg[3]);
 #else /* !LINUX */
@@ -251,6 +272,20 @@ struct tcb *tcp;
 	return 0;
 }
 
+#ifdef LINUX
+int sys_semtimedop(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%lu", tcp->u_arg[0]);
+		tprintf(", %#lx", tcp->u_arg[3]);
+		tprintf(", %lu, ", tcp->u_arg[1]);
+		printtv(tcp, tcp->u_arg[5]);
+	}
+	return 0;
+}
+#endif
+
 int sys_semget(tcp)
 struct tcb *tcp;
 {
@@ -261,9 +296,9 @@ struct tcb *tcp;
 			tprintf("IPC_PRIVATE");
 		tprintf(", %lu", tcp->u_arg[1]);
 		tprintf(", ");
-		if (printflags(resource_flags, tcp->u_arg[2]) != 0)
+		if (printflags(resource_flags, tcp->u_arg[2] & ~0777) != 0)
 			tprintf("|");
-		tprintf("%#lo", tcp->u_arg[2] & 0666);
+		tprintf("%#lo", tcp->u_arg[2] & 0777);
 	}
 	return 0;
 }
@@ -274,7 +309,7 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		tprintf("%lu", tcp->u_arg[0]);
 		tprintf(", %lu, ", tcp->u_arg[1]);
-		printxval(semctl_flags, tcp->u_arg[2], "SEM_???");
+		PRINTCTL(semctl_flags, tcp->u_arg[2], "SEM_???");
 		tprintf(", %#lx", tcp->u_arg[3]);
 	}
 	return 0;
@@ -290,9 +325,9 @@ struct tcb *tcp;
 			tprintf("IPC_PRIVATE");
 		tprintf(", %lu", tcp->u_arg[1]);
 		tprintf(", ");
-		if (printflags(resource_flags, tcp->u_arg[2]) != 0)
+		if (printflags(resource_flags, tcp->u_arg[2] & ~0777) != 0)
 			tprintf("|");
-		tprintf("%#lo", tcp->u_arg[2] & 0666);
+		tprintf("%#lo", tcp->u_arg[2] & 0777);
 	}
 	return 0;
 }
@@ -302,7 +337,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		tprintf("%lu, ", tcp->u_arg[0]);
-		printxval(shmctl_flags, tcp->u_arg[1], "SHM_???");
+		PRINTCTL(shmctl_flags, tcp->u_arg[1], "SHM_???");
 #ifdef LINUX
 		tprintf(", %#lx", tcp->u_arg[3]);
 #else /* !LINUX */
@@ -357,3 +392,101 @@ struct tcb *tcp;
 }
 
 #endif /* defined(LINUX) || defined(SUNOS4) || defined(FREEBSD) */
+
+#ifdef LINUX
+int sys_mq_open(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		/* flags */
+		printflags(openmodes, tcp->u_arg[1] + 1);
+		if (tcp->u_arg[1] & O_CREAT) {
+# ifndef HAVE_MQUEUE_H
+			tprintf(", %lx", tcp->u_arg[2]);
+# else
+			struct mq_attr attr;
+			/* mode */
+			tprintf(", %#lo, ", tcp->u_arg[2]);
+			if (umove(tcp, tcp->u_arg[3], &attr) < 0)
+				tprintf("{ ??? }");
+			else
+				tprintf("{mq_maxmsg=%ld, mq_msgsize=%ld}",
+					attr.mq_maxmsg, attr.mq_msgsize);
+# endif
+		}
+	}
+	return 0;
+}
+
+int sys_mq_timedsend(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
+		tprintf(", %lu, %ld, ", tcp->u_arg[2], tcp->u_arg[3]);
+		printtv(tcp, tcp->u_arg[4]);
+	}
+	return 0;
+}
+
+int sys_mq_timedreceive(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp))
+		tprintf("%ld, ", tcp->u_arg[0]);
+	else {
+		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
+		tprintf(", %lu, %ld, ", tcp->u_arg[2], tcp->u_arg[3]);
+		printtv(tcp, tcp->u_arg[4]);
+	}
+	return 0;
+}
+
+int sys_mq_notify(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printsigevent(tcp, tcp->u_arg[1]);
+	}
+	return 0;
+}
+
+static void printmqattr(tcp, addr)
+struct tcb *tcp;
+long addr;
+{
+	if (addr == 0)
+		tprintf("NULL");
+	else {
+# ifndef HAVE_MQUEUE_H
+		tprintf("%#lx", addr);
+# else
+		struct mq_attr attr;
+		if (umove(tcp, addr, &attr) < 0) {
+			tprintf("{...}");
+			return;
+		}
+		tprintf("{mq_flags=");
+		printflags(openmodes, attr.mq_flags + 1);
+		tprintf(", mq_maxmsg=%ld, mq_msgsize=%ld, mq_curmsg=%ld}",
+			attr.mq_maxmsg, attr.mq_msgsize, attr.mq_curmsgs);
+# endif
+	}
+}
+
+int sys_mq_getsetattr(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printmqattr(tcp, tcp->u_arg[1]);
+		tprintf(", ");
+	} else
+		printmqattr(tcp, tcp->u_arg[2]);
+	return 0;
+}
+#endif
