@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: net.c,v 1.40 2004/07/12 07:44:08 roland Exp $
+ *	$Id: net.c,v 1.44 2004/10/06 22:11:51 roland Exp $
  */
 
 #include "defs.h"
@@ -116,7 +116,7 @@
 #define IPPROTO_MAX IPPROTO_MAX
 #endif
 
-static struct xlat domains[] = {
+static const struct xlat domains[] = {
 #ifdef PF_AAL5
 	{ PF_AAL5,	"PF_AAL5"	},
 #endif
@@ -218,7 +218,7 @@ static struct xlat domains[] = {
 #endif
 	{ 0,		NULL		},
 };
-struct xlat addrfams[] = {
+const struct xlat addrfams[] = {
 #ifdef AF_APPLETALK
 	{ AF_APPLETALK,	"AF_APPLETALK"	},
 #endif
@@ -314,7 +314,7 @@ struct xlat addrfams[] = {
 #endif
 	{ 0,		NULL		},
 };
-static struct xlat socktypes[] = {
+static const struct xlat socktypes[] = {
 	{ SOCK_STREAM,	"SOCK_STREAM"	},
 	{ SOCK_DGRAM,	"SOCK_DGRAM"	},
 #ifdef SOCK_RAW
@@ -331,7 +331,7 @@ static struct xlat socktypes[] = {
 #endif
 	{ 0,		NULL		},
 };
-static struct xlat socketlayers[] = {
+static const struct xlat socketlayers[] = {
 #if defined(SOL_IP)
 	{ SOL_IP,	"SOL_IP"	},
 #endif
@@ -393,7 +393,7 @@ static struct xlat socketlayers[] = {
 };
 /*** WARNING: DANGER WILL ROBINSON: NOTE "socketlayers" array above
      falls into "protocols" array below!!!!   This is intended!!! ***/
-static struct xlat protocols[] = {
+static const struct xlat protocols[] = {
 	{ IPPROTO_IP,	"IPPROTO_IP"	},
 	{ IPPROTO_ICMP,	"IPPROTO_ICMP"	},
 	{ IPPROTO_TCP,	"IPPROTO_TCP"	},
@@ -436,7 +436,7 @@ static struct xlat protocols[] = {
 #endif
 	{ 0,		NULL		},
 };
-static struct xlat msg_flags[] = {
+static const struct xlat msg_flags[] = {
 	{ MSG_OOB,	"MSG_OOB"	},
 #ifdef MSG_DONTROUTE
 	{ MSG_DONTROUTE,"MSG_DONTROUTE"	},
@@ -492,105 +492,132 @@ static struct xlat msg_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat sockoptions[] = {
-#ifdef SO_PEERCRED
-	{ SO_PEERCRED,	"SO_PEERCRED"	},
-#endif
-#ifdef SO_PASSCRED
-	{ SO_PASSCRED,	"SO_PASSCRED"	},
-#endif
-#ifdef SO_DEBUG
-	{ SO_DEBUG,	"SO_DEBUG"	},
-#endif
-#ifdef SO_REUSEADDR
-	{ SO_REUSEADDR,	"SO_REUSEADDR"	},
-#endif
-#ifdef SO_KEEPALIVE
-	{ SO_KEEPALIVE,	"SO_KEEPALIVE"	},
-#endif
-#ifdef SO_DONTROUTE
-	{ SO_DONTROUTE,	"SO_DONTROUTE"	},
-#endif
-#ifdef SO_BROADCAST
-	{ SO_BROADCAST,	"SO_BROADCAST"	},
-#endif
-#ifdef SO_LINGER
-	{ SO_LINGER,	"SO_LINGER"	},
-#endif
-#ifdef SO_OOBINLINE
-	{ SO_OOBINLINE,	"SO_OOBINLINE"	},
-#endif
-#ifdef SO_TYPE
-	{ SO_TYPE,	"SO_TYPE"	},
-#endif
-#ifdef SO_ERROR
-	{ SO_ERROR,	"SO_ERROR"	},
-#endif
-#ifdef SO_SNDBUF
-	{ SO_SNDBUF,	"SO_SNDBUF"	},
-#endif
-#ifdef SO_RCVBUF
-	{ SO_RCVBUF,	"SO_RCVBUF"	},
-#endif
-#ifdef SO_NO_CHECK
-	{ SO_NO_CHECK,	"SO_NO_CHECK"	},
-#endif
-#ifdef SO_PRIORITY
-	{ SO_PRIORITY,	"SO_PRIORITY"	},
-#endif
+static const struct xlat sockoptions[] = {
 #ifdef SO_ACCEPTCONN
-	{ SO_ACCEPTCONN,"SO_ACCEPTCONN"	},
-#endif
-#ifdef SO_USELOOPBACK
-	{ SO_USELOOPBACK,"SO_USELOOPBACK"},
-#endif
-#ifdef SO_SNDLOWAT
-	{ SO_SNDLOWAT,	"SO_SNDLOWAT"	},
-#endif
-#ifdef SO_RCVLOWAT
-	{ SO_RCVLOWAT,	"SO_RCVLOWAT"	},
-#endif
-#ifdef SO_SNDTIMEO
-	{ SO_SNDTIMEO,	"SO_SNDTIMEO"	},
-#endif
-#ifdef SO_RCVTIMEO
-	{ SO_RCVTIMEO,	"SO_RCVTIMEO"	},
-#endif
-#ifdef SO_BSDCOMPAT
-	{ SO_BSDCOMPAT,	"SO_BSDCOMPAT"	},
-#endif
-#ifdef SO_REUSEPORT
-	{ SO_REUSEPORT,	"SO_REUSEPORT"	},
-#endif
-#ifdef SO_ORDREL
-	{ SO_ORDREL,	"SO_ORDREL"	},
-#endif
-#ifdef SO_IMASOCKET
-	{ SO_IMASOCKET,	"SO_IMASOCKET"	},
-#endif
-#ifdef SO_MGMT
-	{ SO_MGMT,	"SO_MGMT"	},
-#endif
-#ifdef SO_LISTENING
-	{ SO_LISTENING,	"SO_LISTENING"	},
-#endif
-#ifdef SO_RDWR
-	{ SO_RDWR,	"SO_RDWR"	},
-#endif
-#ifdef SO_SEMA
-	{ SO_SEMA,	"SO_SEMA"	},
-#endif
-#ifdef SO_PARALLELSVR
-	{ SO_PARALLELSVR,"SO_PARALLELSVR"},
-#endif
-#ifdef SO_PROTOTYPE
-	{ SO_PROTOTYPE,	"SO_PROTOTYPE"	},
+	{ SO_ACCEPTCONN,	"SO_ACCEPTCONN"	},
 #endif
 #ifdef SO_ALLRAW
 	{ SO_ALLRAW,	"SO_ALLRAW"	},
 #endif
+#ifdef SO_ATTACH_FILTER
+	{ SO_ATTACH_FILTER,	"SO_ATTACH_FILTER"	},
+#endif
+#ifdef SO_BINDTODEVICE
+	{ SO_BINDTODEVICE,	"SO_BINDTODEVICE"	},
+#endif
+#ifdef SO_BROADCAST
+	{ SO_BROADCAST,	"SO_BROADCAST"	},
+#endif
+#ifdef SO_BSDCOMPAT
+	{ SO_BSDCOMPAT,	"SO_BSDCOMPAT"	},
+#endif
+#ifdef SO_DEBUG
+	{ SO_DEBUG,	"SO_DEBUG"	},
+#endif
+#ifdef SO_DETACH_FILTER
+	{ SO_DETACH_FILTER,	"SO_DETACH_FILTER"	},
+#endif
+#ifdef SO_DONTROUTE
+	{ SO_DONTROUTE,	"SO_DONTROUTE"	},
+#endif
+#ifdef SO_ERROR
+	{ SO_ERROR,	"SO_ERROR"	},
+#endif
 #ifdef SO_ICS
 	{ SO_ICS,	"SO_ICS"	},
+#endif
+#ifdef SO_IMASOCKET
+	{ SO_IMASOCKET,	"SO_IMASOCKET"	},
+#endif
+#ifdef SO_KEEPALIVE
+	{ SO_KEEPALIVE,	"SO_KEEPALIVE"	},
+#endif
+#ifdef SO_LINGER
+	{ SO_LINGER,	"SO_LINGER"	},
+#endif
+#ifdef SO_LISTENING
+	{ SO_LISTENING,	"SO_LISTENING"	},
+#endif
+#ifdef SO_MGMT
+	{ SO_MGMT,	"SO_MGMT"	},
+#endif
+#ifdef SO_NO_CHECK
+	{ SO_NO_CHECK,	"SO_NO_CHECK"	},
+#endif
+#ifdef SO_OOBINLINE
+	{ SO_OOBINLINE,	"SO_OOBINLINE"	},
+#endif
+#ifdef SO_ORDREL
+	{ SO_ORDREL,	"SO_ORDREL"	},
+#endif
+#ifdef SO_PARALLELSVR
+	{ SO_PARALLELSVR,	"SO_PARALLELSVR"	},
+#endif
+#ifdef SO_PASSCRED
+	{ SO_PASSCRED,	"SO_PASSCRED"	},
+#endif
+#ifdef SO_PEERCRED
+	{ SO_PEERCRED,	"SO_PEERCRED"	},
+#endif
+#ifdef SO_PEERNAME
+	{ SO_PEERNAME,	"SO_PEERNAME"	},
+#endif
+#ifdef SO_PEERSEC
+	{ SO_PEERSEC,	"SO_PEERSEC"	},
+#endif
+#ifdef SO_PRIORITY
+	{ SO_PRIORITY,	"SO_PRIORITY"	},
+#endif
+#ifdef SO_PROTOTYPE
+	{ SO_PROTOTYPE,	"SO_PROTOTYPE"	},
+#endif
+#ifdef SO_RCVBUF
+	{ SO_RCVBUF,	"SO_RCVBUF"	},
+#endif
+#ifdef SO_RCVLOWAT
+	{ SO_RCVLOWAT,	"SO_RCVLOWAT"	},
+#endif
+#ifdef SO_RCVTIMEO
+	{ SO_RCVTIMEO,	"SO_RCVTIMEO"	},
+#endif
+#ifdef SO_RDWR
+	{ SO_RDWR,	"SO_RDWR"	},
+#endif
+#ifdef SO_REUSEADDR
+	{ SO_REUSEADDR,	"SO_REUSEADDR"	},
+#endif
+#ifdef SO_REUSEPORT
+	{ SO_REUSEPORT,	"SO_REUSEPORT"	},
+#endif
+#ifdef SO_SECURITY_AUTHENTICATION
+	{ SO_SECURITY_AUTHENTICATION,"SO_SECURITY_AUTHENTICATION"},
+#endif
+#ifdef SO_SECURITY_ENCRYPTION_NETWORK
+	{ SO_SECURITY_ENCRYPTION_NETWORK,"SO_SECURITY_ENCRYPTION_NETWORK"},
+#endif
+#ifdef SO_SECURITY_ENCRYPTION_TRANSPORT
+	{ SO_SECURITY_ENCRYPTION_TRANSPORT,"SO_SECURITY_ENCRYPTION_TRANSPORT"},
+#endif
+#ifdef SO_SEMA
+	{ SO_SEMA,	"SO_SEMA"	},
+#endif
+#ifdef SO_SNDBUF
+	{ SO_SNDBUF,	"SO_SNDBUF"	},
+#endif
+#ifdef SO_SNDLOWAT
+	{ SO_SNDLOWAT,	"SO_SNDLOWAT"	},
+#endif
+#ifdef SO_SNDTIMEO
+	{ SO_SNDTIMEO,	"SO_SNDTIMEO"	},
+#endif
+#ifdef SO_TIMESTAMP
+	{ SO_TIMESTAMP,	"SO_TIMESTAMP"	},
+#endif
+#ifdef SO_TYPE
+	{ SO_TYPE,	"SO_TYPE"	},
+#endif
+#ifdef SO_USELOOPBACK
+	{ SO_USELOOPBACK,	"SO_USELOOPBACK"	},
 #endif
 	{ 0,		NULL		},
 };
@@ -600,7 +627,7 @@ static struct xlat sockoptions[] = {
 #endif
 
 #ifdef SOL_IP
-static struct xlat sockipoptions[] = {
+static const struct xlat sockipoptions[] = {
 #ifdef IP_TOS
 	{ IP_TOS,		"IP_TOS"		},
 #endif
@@ -672,20 +699,89 @@ static struct xlat sockipoptions[] = {
 #endif
 #ifdef IP_RECVIFINDEX
 	{ IP_RECVIFINDEX,	"IP_RECVIFINDEX"	},
+#endif
+#ifdef IP_MSFILTER
+	{ IP_MSFILTER,		"IP_MSFILTER"		},
+#endif
+#ifdef MCAST_MSFILTER
+	{ MCAST_MSFILTER,	"MCAST_MSFILTER"	},
+#endif
+#ifdef IP_FREEBIND
+	{ IP_FREEBIND,		"IP_FREEBIND"		},
 #endif
 	{ 0,			NULL			},
 };
 #endif /* SOL_IP */
 
+#ifdef SOL_IPV6
+static const struct xlat sockipv6options[] = {
+#ifdef IPV6_ADDRFORM
+	{ IPV6_ADDRFORM,	"IPV6_ADDRFORM"		},
+#endif
+#ifdef MCAST_FILTER
+	{ MCAST_FILTER,		"MCAST_FILTER"		},
+#endif
+#ifdef IPV6_PKTOPTIONS
+	{ IPV6_PKTOPTIONS,	"IPV6_PKTOPTIONS"	},
+#endif
+#ifdef IPV6_MTU
+	{ IPV6_MTU,		"IPV6_MTU"		},
+#endif
+#ifdef IPV6_V6ONLY
+	{ IPV6_V6ONLY,		"IPV6_V6ONLY"		},
+#endif
+#ifdef IPV6_PKTINFO
+	{ IPV6_PKTINFO,		"IPV6_PKTINFO"		},
+#endif
+#ifdef IPV6_HOPLIMIT
+	{ IPV6_HOPLIMIT,	"IPV6_HOPLIMIT"		},
+#endif
+#ifdef IPV6_RTHDR
+	{ IPV6_RTHDR,		"IPV6_RTHDR"		},
+#endif
+#ifdef IPV6_HOPOPTS
+	{ IPV6_HOPOPTS,		"IPV6_HOPOPTS"		},
+#endif
+#ifdef IPV6_DSTOPTS
+	{ IPV6_DSTOPTS,		"IPV6_DSTOPTS"		},
+#endif
+#ifdef IPV6_FLOWINFO
+	{ IPV6_FLOWINFO,	"IPV6_FLOWINFO"		},
+#endif
+#ifdef IPV6_UNICAST_HOPS
+	{ IPV6_UNICAST_HOPS,	"IPV6_UNICAST_HOPS"	},
+#endif
+#ifdef IPV6_MULTICAST_HOPS
+	{ IPV6_MULTICAST_HOPS,	"IPV6_MULTICAST_HOPS"	},
+#endif
+#ifdef IPV6_MULTICAST_LOOP
+	{ IPV6_MULTICAST_LOOP,	"IPV6_MULTICAST_LOOP"	},
+#endif
+#ifdef IPV6_MULTICAST_IF
+	{ IPV6_MULTICAST_IF,	"IPV6_MULTICAST_IF"	},
+#endif
+#ifdef IPV6_MTU_DISCOVER
+	{ IPV6_MTU_DISCOVER,	"IPV6_MTU_DISCOVER"	},
+#endif
+#ifdef IPV6_RECVERR
+	{ IPV6_RECVERR,		"IPV6_RECVERR"		},
+#endif
+#ifdef IPV6_FLOWINFO_SEND
+	{ IPV6_FLOWINFO_SEND,	"IPV6_FLOWINFO_SEND"	},
+#endif
+	{ 0,			NULL			},
+};
+#endif /* SOL_IPV6 */
+
 #ifdef SOL_IPX
-static struct xlat sockipxoptions[] = {
+static const struct xlat sockipxoptions[] = {
 	{ IPX_TYPE,     "IPX_TYPE"      },
 	{ 0,            NULL            },
 };
 #endif /* SOL_IPX */
 
 #ifdef SOL_RAW
-static struct xlat sockrawoptions[] = {
+static const struct xlat sockrawoptions[] = {
 #if defined(ICMP_FILTER)
 	{ ICMP_FILTER,		"ICMP_FILTER"	},
 #endif
@@ -694,7 +790,7 @@ static struct xlat sockrawoptions[] = {
 #endif /* SOL_RAW */
 
 #ifdef SOL_PACKET
-static struct xlat sockpacketoptions[] = {
+static const struct xlat sockpacketoptions[] = {
 	{ PACKET_ADD_MEMBERSHIP,	"PACKET_ADD_MEMBERSHIP"	},
 	{ PACKET_DROP_MEMBERSHIP,	"PACKET_DROP_MEMBERSHIP"},
 #if defined(PACKET_RECV_OUTPUT)
@@ -715,7 +811,7 @@ static struct xlat sockpacketoptions[] = {
 #endif
 
 #ifdef SOL_TCP
-static struct xlat socktcpoptions[] = {
+static const struct xlat socktcpoptions[] = {
 	{ TCP_NODELAY,		"TCP_NODELAY"	},
 	{ TCP_MAXSEG,		"TCP_MAXSEG"	},
 #if defined(TCP_CORK)
@@ -756,7 +852,7 @@ static struct xlat socktcpoptions[] = {
 #endif /* SOL_TCP */
 
 #ifdef SOL_RAW
-static struct xlat icmpfilterflags[] = {
+static const struct xlat icmpfilterflags[] = {
 #if defined(ICMP_ECHOREPLY)
 	{ (1<<ICMP_ECHOREPLY),		"ICMP_ECHOREPLY"	},
 #endif
@@ -801,7 +897,7 @@ static struct xlat icmpfilterflags[] = {
 #endif /* SOL_RAW */
 
 #if defined(AF_PACKET) /* from e.g. linux/if_packet.h */
-static struct xlat af_packet_types[] = {
+static const struct xlat af_packet_types[] = {
 #if defined(PACKET_HOST)
 	{ PACKET_HOST,			"PACKET_HOST"		},
 #endif
@@ -970,6 +1066,62 @@ int addrlen;
 }
 
 #if HAVE_SENDMSG
+static const struct xlat scmvals[] = {
+#ifdef SCM_RIGHTS
+	{ SCM_RIGHTS,		"SCM_RIGHTS"		},
+#endif
+#ifdef SCM_CREDENTIALS
+	{ SCM_CREDENTIALS,	"SCM_CREDENTIALS"	},
+#endif
+	{ 0,			NULL			}
+};
+
+static void
+printcmsghdr(tcp, addr, len)
+struct tcb *tcp;
+unsigned long addr;
+unsigned long len;
+{
+	union {
+		char msg_control[len];
+		struct cmsghdr cmsg;
+	} u;
+	if (umoven(tcp, addr, len, u.msg_control) < 0) {
+		tprintf(", msg_control=%#lx", addr);
+		return;
+	}
+
+	tprintf(", {cmsg_len=%zu, cmsg_level=", u.cmsg.cmsg_len);
+	printxval(socketlayers, u.cmsg.cmsg_level, "SOL_???");
+	tprintf(", cmsg_type=");
+
+	if (u.cmsg.cmsg_level == SOL_SOCKET) {
+		printxval(scmvals, u.cmsg.cmsg_type, "SCM_???");
+			
+		if (u.cmsg.cmsg_type == SCM_RIGHTS) {
+			int *fds = (int *) CMSG_DATA (&u.cmsg);
+			int first = 1;
+			tprintf(", {");
+			while ((char *) fds < (u.msg_control
+					       + u.cmsg.cmsg_len)) {
+				if (!first)
+					tprintf(", ");
+				tprintf("%d", *fds++);
+				first = 0;
+			}
+			tprintf("}}");
+			return;
+		}
+		if (u.cmsg.cmsg_type == SCM_CREDENTIALS
+		    && CMSG_LEN(sizeof(struct ucred)) <= u.cmsg.cmsg_len) {
+			struct ucred *uc = (struct ucred *) CMSG_DATA (&u.cmsg);
+			tprintf("{pid=%ld, uid=%ld, gid=%ld}}",
+				(long)uc->pid, (long)uc->uid, (long)uc->gid);
+			return;
+		}
+	}
+	tprintf(", ...}");
+}
 
 static void
 printmsghdr(tcp, addr)
@@ -991,7 +1143,8 @@ long addr;
 #ifdef HAVE_STRUCT_MSGHDR_MSG_CONTROL
 	tprintf(", msg_controllen=%lu", (unsigned long)msg.msg_controllen);
 	if (msg.msg_controllen)
-		tprintf(", msg_control=%#lx, ", (unsigned long) msg.msg_control);
+		printcmsghdr(tcp, (unsigned long) msg.msg_control,
+			     msg.msg_controllen);
 	tprintf(", msg_flags=");
 	if (printflags(msg_flags, msg.msg_flags)==0)
 		tprintf("0");
@@ -1378,6 +1531,11 @@ struct tcb *tcp;
 			printxval(sockipoptions, tcp->u_arg[2], "IP_???");
 			break;
 #endif
+#ifdef SOL_IPV6
+		case SOL_IPV6:
+			printxval(sockipv6options, tcp->u_arg[2], "IPV6_???");
+			break;
+#endif
 #ifdef SOL_IPX
 		case SOL_IPX:
 			printxval(sockipxoptions, tcp->u_arg[2], "IPX_???");
@@ -1505,6 +1663,11 @@ int len;
 		printxval(sockipoptions, name, "IP_???");
 		break;
 #endif
+#ifdef SOL_IPV6
+	    case SOL_IPV6:
+		printxval(sockipv6options, name, "IPV6_???");
+		break;
+#endif
 #ifdef SOL_IPX
 	    case SOL_IPX:
 		printxval(sockipxoptions, name, "IPX_???");
@@ -1609,7 +1772,7 @@ struct tcb *tcp;
 
 #if UNIXWARE >= 7
 
-static struct xlat sock_version[] = {
+static const struct xlat sock_version[] = {
 	{ __NETLIB_UW211_SVR4,	"UW211_SVR4" },
 	{ __NETLIB_UW211_XPG4,	"UW211_XPG4" },
 	{ __NETLIB_GEMINI_SVR4,	"GEMINI_SVR4" },
