@@ -34,7 +34,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: process.c,v 1.86 2004/07/12 07:44:09 roland Exp $
+ *	$Id: process.c,v 1.90 2004/10/19 23:33:47 roland Exp $
  */
 
 #include "defs.h"
@@ -143,7 +143,7 @@
 
 /* WTA: this was `&& !defined(LINUXSPARC)', this seems unneeded though? */
 #if defined(HAVE_PRCTL)
-static struct xlat prctl_options[] = {
+static const struct xlat prctl_options[] = {
 #ifdef PR_MAXPROCS
 	{ PR_MAXPROCS,		"PR_MAXPROCS"		},
 #endif
@@ -510,7 +510,7 @@ struct tcb *tcp;
 #define CLONE_UNTRACED		0x00800000	/* set if the tracing process can't force CLONE_PTRACE on this clone */
 #define CLONE_CHILD_SETTID	0x01000000	/* set the TID in the child */
 
-static struct xlat clone_flags[] = {
+static const struct xlat clone_flags[] = {
     { CLONE_VM,		"CLONE_VM"	},
     { CLONE_FS,		"CLONE_FS"	},
     { CLONE_FILES,	"CLONE_FILES"	},
@@ -1442,7 +1442,7 @@ struct tcb *tcp;
 #include <sys/privilege.h>
 
 
-static struct xlat procpriv_cmds [] = {
+static const struct xlat procpriv_cmds [] = {
 	{ SETPRV,	"SETPRV"	},
 	{ CLRPRV,	"CLRPRV"	},
 	{ PUTPRV,	"PUTPRV"	},
@@ -1452,7 +1452,7 @@ static struct xlat procpriv_cmds [] = {
 };
 
 
-static struct xlat procpriv_priv [] = {
+static const struct xlat procpriv_priv [] = {
 	{ P_OWNER,	"P_OWNER"	},
 	{ P_AUDIT,	"P_AUDIT"	},
 	{ P_COMPAT,	"P_COMPAT"	},
@@ -1485,7 +1485,7 @@ static struct xlat procpriv_priv [] = {
 };
 
 
-static struct xlat procpriv_type [] = {
+static const struct xlat procpriv_type [] = {
 	{ PS_FIX,	"PS_FIX"	},
 	{ PS_INH,	"PS_INH"	},
 	{ PS_MAX,	"PS_MAX"	},
@@ -1499,7 +1499,7 @@ printpriv(tcp, addr, len, opt)
 struct tcb *tcp;
 long addr;
 int len;
-struct xlat *opt;
+const struct xlat *opt;
 {
 	priv_t buf [128];
 	int max = verbose (tcp) ? sizeof buf / sizeof buf [0] : 10;
@@ -1743,7 +1743,7 @@ struct tcb *tcp;
 #endif
 #endif /* LINUX */
 
-static struct xlat wait4_options[] = {
+static const struct xlat wait4_options[] = {
 	{ WNOHANG,	"WNOHANG"	},
 #ifndef WSTOPPED
 	{ WUNTRACED,	"WUNTRACED"	},
@@ -1775,6 +1775,20 @@ static struct xlat wait4_options[] = {
 	{ 0,		NULL		},
 };
 
+#if !defined WCOREFLAG && defined WCOREFLG
+# define WCOREFLAG WCOREFLG
+#endif
+#ifndef WCOREFLAG
+#define WCOREFLAG 0x80
+#endif
+
+#ifndef W_STOPCODE
+#define W_STOPCODE(sig)		((sig) << 8 | 0x7f)
+#endif
+#ifndef W_EXITCODE
+#define W_EXITCODE(ret, sig)	((ret) << 8 | (sig))
+#endif
+
 static int
 printstatus(status)
 int status;
@@ -1868,8 +1882,9 @@ int bitness;
 }
 
 int
-internal_wait(tcp)
+internal_wait(tcp, flagarg)
 struct tcb *tcp;
+int flagarg;
 {
 	int got_kids;
 
@@ -1895,7 +1910,7 @@ struct tcb *tcp;
 
 		/* ??? WTA: fix bug with hanging children */
 
-		if (!(tcp->u_arg[2] & WNOHANG)) {
+		if (!(tcp->u_arg[flagarg] & WNOHANG)) {
 			/*
 			 * There are traced children.  We'll make the parent
 			 * block to avoid a false ECHILD error due to our
@@ -1936,7 +1951,7 @@ struct tcb *tcp;
 		}
 	}
 	if (exiting(tcp) && tcp->u_error == ECHILD && got_kids) {
-		if (tcp->u_arg[2] & WNOHANG) {
+		if (tcp->u_arg[flagarg] & WNOHANG) {
 			/* We must force a fake result of 0 instead of
 			   the ECHILD error.  */
 			extern int force_result();
@@ -2012,16 +2027,26 @@ struct tcb *tcp;
 }
 #endif
 
-#ifdef SVR4
+#if defined SVR4 || defined LINUX
 
-static struct xlat waitid_types[] = {
+static const struct xlat waitid_types[] = {
 	{ P_PID,	"P_PID"		},
+#ifdef P_PPID
 	{ P_PPID,	"P_PPID"	},
+#endif
 	{ P_PGID,	"P_PGID"	},
+#ifdef P_SID
 	{ P_SID,	"P_SID"		},
+#endif
+#ifdef P_CID
 	{ P_CID,	"P_CID"		},
+#endif
+#ifdef P_UID
 	{ P_UID,	"P_UID"		},
+#endif
+#ifdef P_GID
 	{ P_GID,	"P_GID"		},
+#endif
 	{ P_ALL,	"P_ALL"		},
 #ifdef P_LWPID
 	{ P_LWPID,	"P_LWPID"	},
@@ -2039,11 +2064,6 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		printxval(waitid_types, tcp->u_arg[0], "P_???");
 		tprintf(", %ld, ", tcp->u_arg[1]);
-		if (tcp->nchildren > 0) {
-			/* There are traced children */
-			tcp->flags |= TCB_SUSPENDED;
-			tcp->waitpid = tcp->u_arg[0];
-		}
 	}
 	else {
 		/* siginfo */
@@ -2060,11 +2080,21 @@ struct tcb *tcp;
 		tprintf(", ");
 		if (!printflags(wait4_options, tcp->u_arg[3]))
 			tprintf("0");
+		if (tcp->u_nargs > 4) {
+			/* usage */
+			tprintf(", ");
+			if (!tcp->u_arg[4])
+				tprintf("NULL");
+			else if (tcp->u_error)
+				tprintf("%#lx", tcp->u_arg[4]);
+			else
+				printrusage(tcp, tcp->u_arg[4]);
+		}
 	}
 	return 0;
 }
 
-#endif /* SVR4 */
+#endif /* SVR4 or LINUX */
 
 int
 sys_alarm(tcp)
@@ -2109,7 +2139,7 @@ struct tcb *tcp;
 
 #ifndef SVR4
 
-static struct xlat ptrace_cmds[] = {
+static const struct xlat ptrace_cmds[] = {
 #ifndef FREEBSD
 	{ PTRACE_TRACEME,	"PTRACE_TRACEME"	},
 	{ PTRACE_PEEKTEXT,	"PTRACE_PEEKTEXT",	},
@@ -2198,7 +2228,7 @@ static struct xlat ptrace_cmds[] = {
 #ifndef SUNOS4_KERNEL_ARCH_KLUDGE
 static
 #endif /* !SUNOS4_KERNEL_ARCH_KLUDGE */
-struct xlat struct_user_offsets[] = {
+const struct xlat struct_user_offsets[] = {
 #ifdef LINUX
 #if defined(S390) || defined(S390X)
 	{ PT_PSWMASK,		"psw_mask"				},
@@ -2881,7 +2911,7 @@ int
 sys_ptrace(tcp)
 struct tcb *tcp;
 {
-	struct xlat *x;
+	const struct xlat *x;
 	long addr;
 
 	if (entering(tcp)) {
@@ -2966,7 +2996,7 @@ struct tcb *tcp;
 #endif /* !SVR4 */
 
 #ifdef LINUX
-static struct xlat futexops[] = {
+static const struct xlat futexops[] = {
 	{ FUTEX_WAIT,	"FUTEX_WAIT" },
 	{ FUTEX_WAKE,	"FUTEX_WAKE" },
 	{ FUTEX_FD,	"FUTEX_FD" },
@@ -3036,7 +3066,7 @@ struct tcb *tcp;
     return 0;
 }
 
-static struct xlat schedulers[] = {
+static const struct xlat schedulers[] = {
 	{ SCHED_OTHER,	"SCHED_OTHER" },
 	{ SCHED_RR,	"SCHED_RR" },
 	{ SCHED_FIFO,	"SCHED_FIFO" },
