@@ -30,7 +30,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: signal.c,v 1.47 2004/07/12 07:44:08 roland Exp $
+ *	$Id: signal.c,v 1.51 2004/10/06 22:11:54 roland Exp $
  */
 
 #include "defs.h"
@@ -153,31 +153,31 @@ struct sigcontext
 #endif
 #endif /* LINUX */
 
-char *signalent0[] = {
+const char *const signalent0[] = {
 #include "signalent.h"
 };
-int nsignals0 = sizeof signalent0 / sizeof signalent0[0];
+const int nsignals0 = sizeof signalent0 / sizeof signalent0[0];
 
 #if SUPPORTED_PERSONALITIES >= 2
-char *signalent1[] = {
+const char *const signalent1[] = {
 #include "signalent1.h"
 };
-int nsignals1 = sizeof signalent1 / sizeof signalent1[0];
+const int nsignals1 = sizeof signalent1 / sizeof signalent1[0];
 #endif /* SUPPORTED_PERSONALITIES >= 2 */
 
 #if SUPPORTED_PERSONALITIES >= 3
-char *signalent2[] = {
+const char *const signalent2[] = {
 #include "signalent2.h"
 };
-int nsignals2 = sizeof signalent2 / sizeof signalent2[0];
+const int nsignals2 = sizeof signalent2 / sizeof signalent2[0];
 #endif /* SUPPORTED_PERSONALITIES >= 3 */
 
-char **signalent;
+const char *const *signalent;
 int nsignals;
 
 #if defined(SUNOS4) || defined(FREEBSD)
 
-static struct xlat sigvec_flags[] = {
+static const struct xlat sigvec_flags[] = {
 	{ SV_ONSTACK,	"SV_ONSTACK"	},
 	{ SV_INTERRUPT,	"SV_INTERRUPT"	},
 	{ SV_RESETHAND,	"SV_RESETHAND"	},
@@ -197,7 +197,7 @@ static struct xlat sigvec_flags[] = {
 # endif
 #endif
 
-static struct xlat sigact_flags[] = {
+static const struct xlat sigact_flags[] = {
 #ifdef SA_RESTORER
 	{ SA_RESTORER,	"SA_RESTORER"	},
 #endif
@@ -240,7 +240,7 @@ static struct xlat sigact_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat sigprocmaskcmds[] = {
+static const struct xlat sigprocmaskcmds[] = {
 	{ SIG_BLOCK,	"SIG_BLOCK"	},
 	{ SIG_UNBLOCK,	"SIG_UNBLOCK"	},
 	{ SIG_SETMASK,	"SIG_SETMASK"	},
@@ -262,7 +262,7 @@ static struct xlat sigprocmaskcmds[] = {
 #endif
 #endif
 
-char *
+const char *
 signame(sig)
 int sig;
 {
@@ -528,7 +528,7 @@ typedef struct siginfo
 
 #if defined (SVR4) || defined (LINUX)
 
-static struct xlat siginfo_codes[] = {
+static const struct xlat siginfo_codes[] = {
 #ifdef SI_NOINFO
 	{ SI_NOINFO,	"SI_NOINFO"	},
 #endif
@@ -559,7 +559,7 @@ static struct xlat siginfo_codes[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat sigill_codes[] = {
+static const struct xlat sigill_codes[] = {
 	{ ILL_ILLOPC,	"ILL_ILLOPC"	},
 	{ ILL_ILLOPN,	"ILL_ILLOPN"	},
 	{ ILL_ILLADR,	"ILL_ILLADR"	},
@@ -571,7 +571,7 @@ static struct xlat sigill_codes[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat sigfpe_codes[] = {
+static const struct xlat sigfpe_codes[] = {
 	{ FPE_INTDIV,	"FPE_INTDIV"	},
 	{ FPE_INTOVF,	"FPE_INTOVF"	},
 	{ FPE_FLTDIV,	"FPE_FLTDIV"	},
@@ -583,13 +583,13 @@ static struct xlat sigfpe_codes[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat sigtrap_codes[] = {
+static const struct xlat sigtrap_codes[] = {
 	{ TRAP_BRKPT,	"TRAP_BRKPT"	},
 	{ TRAP_TRACE,	"TRAP_TRACE"	},
 	{ 0,		NULL		},
 };
 
-static struct xlat sigchld_codes[] = {
+static const struct xlat sigchld_codes[] = {
 	{ CLD_EXITED,	"CLD_EXITED"	},
 	{ CLD_KILLED,	"CLD_KILLED"	},
 	{ CLD_DUMPED,	"CLD_DUMPED"	},
@@ -599,7 +599,7 @@ static struct xlat sigchld_codes[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat sigpoll_codes[] = {
+static const struct xlat sigpoll_codes[] = {
 	{ POLL_IN,	"POLL_IN"	},
 	{ POLL_OUT,	"POLL_OUT"	},
 	{ POLL_MSG,	"POLL_MSG"	},
@@ -609,7 +609,7 @@ static struct xlat sigpoll_codes[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat sigprof_codes[] = {
+static const struct xlat sigprof_codes[] = {
 #ifdef PROF_SIG
 	{ PROF_SIG,	"PROF_SIG"	},
 #endif
@@ -617,7 +617,7 @@ static struct xlat sigprof_codes[] = {
 };
 
 #ifdef SIGEMT
-static struct xlat sigemt_codes[] = {
+static const struct xlat sigemt_codes[] = {
 #ifdef EMT_TAGOVF
 	{ EMT_TAGOVF,	"EMT_TAGOVF"	},
 #endif
@@ -625,13 +625,13 @@ static struct xlat sigemt_codes[] = {
 };
 #endif
 
-static struct xlat sigsegv_codes[] = {
+static const struct xlat sigsegv_codes[] = {
 	{ SEGV_MAPERR,	"SEGV_MAPERR"	},
 	{ SEGV_ACCERR,	"SEGV_ACCERR"	},
 	{ 0,		NULL		},
 };
 
-static struct xlat sigbus_codes[] = {
+static const struct xlat sigbus_codes[] = {
 	{ BUS_ADRALN,	"BUS_ADRALN"	},
 	{ BUS_ADRERR,	"BUS_ADRERR"	},
 	{ BUS_OBJERR,	"BUS_OBJERR"	},
@@ -643,7 +643,7 @@ printsiginfo(sip, verbose)
 siginfo_t *sip;
 int verbose;
 {
-	char *code;
+	const char *code;
 
 	if (sip->si_signo == 0) {
 		tprintf ("{}");
@@ -793,9 +793,9 @@ parse_sigset_t (const char *str, sigset_t *set)
 		if (*p >= '0' && *p <= '9')
 			digit = *p - '0';
 		else if (*p >= 'a' && *p <= 'f')
-			digit = *p - 'a';
+			digit = *p - 'a' + 10;
 		else if (*p >= 'A' && *p <= 'F')
-			digit = *p - 'A';
+			digit = *p - 'A' + 10;
 		else
 			break;
 		if (digit & 1)
@@ -1493,7 +1493,7 @@ struct tcb *tcp;
 	return 0;
 }
 #ifndef FREEBSD
-static struct xlat ucontext_flags[] = {
+static const struct xlat ucontext_flags[] = {
 	{ UC_SIGMASK,	"UC_SIGMASK"	},
 	{ UC_STACK,	"UC_STACK"	},
 	{ UC_CPU,	"UC_CPU"	},
@@ -1525,7 +1525,7 @@ typedef struct
 #define stack_t struct sigaltstack
 #endif
 
-static struct xlat sigaltstack_flags[] = {
+static const struct xlat sigaltstack_flags[] = {
 	{ SS_ONSTACK,	"SS_ONSTACK"	},
 	{ SS_DISABLE,	"SS_DISABLE"	},
 	{ 0,		NULL		},
