@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: strace.c,v 1.23 2001/08/03 11:43:35 wichert Exp $
+ *	$Id: strace.c,v 1.60 2004/10/20 02:04:15 roland Exp $
  */
 
 #include <sys/types.h>
@@ -43,6 +43,12 @@
 #include <pwd.h>
 #include <grp.h>
 #include <string.h>
+#include <limits.h>
+#include <dirent.h>
+
+#if defined(IA64) && defined(LINUX)
+# include <asm/ptrace_offsets.h>
+#endif
 
 #ifdef USE_PROCFS
 #include <poll.h>
@@ -62,6 +68,9 @@ int rflag = 0, tflag = 0, dtime = 0, cflag = 0;
 int iflag = 0, xflag = 0, qflag = 0;
 int pflag_seen = 0;
 
+/* Sometimes we want to print only succeeding syscalls. */
+int not_failing_only = 0;
+
 char *username = NULL;
 uid_t run_uid;
 gid_t run_gid;
@@ -70,13 +79,11 @@ int acolumn = DEFAULT_ACOLUMN;
 int max_strlen = DEFAULT_STRLEN;
 char *outfname = NULL;
 FILE *outf;
-struct tcb tcbtab[MAX_PROCS];
-int nprocs;
+struct tcb **tcbtab;
+unsigned int nprocs, tcbtabsize;
 char *progname;
-extern char version[];
 extern char **environ;
 
-static struct tcb *pid2tcb P((int pid));
 static int trace P((void));
 static void cleanup P((void));
 static void interrupt P((int sig));
@@ -97,7 +104,7 @@ static int interrupted;
 static struct tcb *pfd2tcb P((int pfd));
 static void reaper P((int sig));
 static void rebuild_pollv P((void));
-struct pollfd pollv[MAX_PROCS];
+static struct pollfd *pollv;
 
 #ifndef HAVE_POLLABLE_PROCFS
 
@@ -129,8 +136,10 @@ int exitval;
 {
 	fprintf(ofp, "\
 usage: strace [-dffhiqrtttTvVxx] [-a column] [-e expr] ... [-o file]\n\
-              [-p pid] ... [-s strsize] [-u username] [command [arg ...]]\n\
-   or: strace -c [-e expr] ... [-O overhead] [-S sortby] [command [arg ...]]\n\
+              [-p pid] ... [-s strsize] [-u username] [-E var=val] ...\n\
+              [command [arg ...]]\n\
+   or: strace -c [-e expr] ... [-O overhead] [-S sortby] [-E var=val] ...\n\
+              [command [arg ...]]\n\
 -c -- count time, calls, and errors for each syscall and report summary\n\
 -f -- follow forks, -ff -- with output into separate files\n\
 -F -- attempt to follow vforks, -h -- print help message\n\
@@ -149,7 +158,12 @@ usage: strace [-dffhiqrtttTvVxx] [-a column] [-e expr] ... [-o file]\n\
 -s strsize -- limit length of print strings to STRSIZE chars (default %d)\n\
 -S sortby -- sort syscall counts by: time, calls, name, nothing (default %s)\n\
 -u username -- run command as username handling setuid and/or setgid\n\
-", DEFAULT_ACOLUMN, DEFAULT_STRLEN, DEFAULT_SORTBY);
+-E var=val -- put var=val in the environment for command\n\
+-E var -- remove var from the environment for command\n\
+" /* this is broken, so don't document it
+-z -- print only succeeding syscalls\n\
+  */
+, DEFAULT_ACOLUMN, DEFAULT_STRLEN, DEFAULT_SORTBY);
 	exit(exitval);
 }
 
@@ -175,6 +189,13 @@ char *argv[];
 
 	static char buf[BUFSIZ];
 
+	/* Allocate the initial tcbtab.  */
+	tcbtabsize = argc;	/* Surely enough for all -p args.  */
+	tcbtab = (struct tcb **) malloc (tcbtabsize * sizeof tcbtab[0]);
+	tcbtab[0] = (struct tcb *) calloc (tcbtabsize, sizeof *tcbtab[0]);
+	for (tcp = tcbtab[0]; tcp < &tcbtab[0][tcbtabsize]; ++tcp)
+		tcbtab[tcp - tcbtab[0]] = &tcbtab[0][tcp - tcbtab[0]];
+
 	progname = argv[0];
 	outf = stderr;
 	interactive = 1;
@@ -185,7 +206,7 @@ char *argv[];
 	set_sortby(DEFAULT_SORTBY);
 	set_personality(DEFAULT_PERSONALITY);
 	while ((c = getopt(argc, argv,
-		"+cdfFhiqrtTvVxa:e:o:O:p:s:S:u:")) != EOF) {
+		"+cdfFhiqrtTvVxza:e:o:O:p:s:S:u:E:")) != EOF) {
 		switch (c) {
 		case 'c':
 			cflag++;
@@ -226,9 +247,12 @@ char *argv[];
 			qualify("abbrev=none");
 			break;
 		case 'V':
-			printf("%s\n", version);
+			printf("%s -- version %s\n", PACKAGE_NAME, VERSION);
 			exit(0);
 			break;
+		case 'z':
+			not_failing_only = 1;
+			break;
 		case 'a':
 			acolumn = atoi(optarg);
 			break;
@@ -242,7 +266,7 @@ char *argv[];
 			set_overhead(atoi(optarg));
 			break;
 		case 'p':
-			if ((pid = atoi(optarg)) == 0) {
+			if ((pid = atoi(optarg)) <= 0) {
 				fprintf(stderr, "%s: Invalid process id: %s\n",
 					progname, optarg);
 				break;
@@ -252,7 +276,7 @@ char *argv[];
 				break;
 			}
 			if ((tcp = alloctcb(pid)) == NULL) {
-				fprintf(stderr, "%s: tcb table full, please recompile strace\n",
+				fprintf(stderr, "%s: out of memory\n",
 					progname);
 				exit(1);
 			}
@@ -268,12 +292,22 @@ char *argv[];
 		case 'u':
 			username = strdup(optarg);
 			break;
+		case 'E':
+			if (putenv(optarg) < 0) {
+				fprintf(stderr, "%s: out of memory\n",
+					progname);
+				exit(1);
+			}
+			break;
 		default:
 			usage(stderr, 1);
 			break;
 		}
 	}
 
+	if (optind == argc && !pflag_seen)
+		usage(stderr, 1);
+
 	/* See if they want to run as another user. */
 	if (username != NULL) {
 		struct passwd *pent;
@@ -301,22 +335,31 @@ char *argv[];
 	setreuid(geteuid(), getuid());
 #endif
 
-	/* See if they want to pipe the output. */
-	if (outfname && (outfname[0] == '|' || outfname[0] == '!')) {
-		if ((outf = popen(outfname + 1, "w")) == NULL) {
-			fprintf(stderr, "%s: can't popen '%s': %s\n",
-				progname, outfname + 1, strerror(errno));
-			exit(1);
-		}
-		free(outfname);
-		outfname = NULL;
-	}
-
 	/* Check if they want to redirect the output. */
 	if (outfname) {
 		long f;
 
-		if ((outf = fopen(outfname, "w")) == NULL) {
+		/* See if they want to pipe the output. */
+		if (outfname[0] == '|' || outfname[0] == '!') {
+			/*
+			 * We can't do the <outfname>.PID funny business
+			 * when using popen, so prohibit it.
+			 */
+			if (followfork > 1) {
+				fprintf(stderr, "\
+%s: piping the output and -ff are mutually exclusive options\n",
+					progname);
+				exit(1);
+			}
+
+			if ((outf = popen(outfname + 1, "w")) == NULL) {
+				fprintf(stderr, "%s: can't popen '%s': %s\n",
+					progname, outfname + 1,
+					strerror(errno));
+				exit(1);
+			}
+		}
+		else if ((outf = fopen(outfname, "w")) == NULL) {
 			fprintf(stderr, "%s: can't fopen '%s': %s\n",
 				progname, outfname, strerror(errno));
 			exit(1);
@@ -337,16 +380,15 @@ char *argv[];
 	setreuid(geteuid(), getuid());
 #endif
 
-	if (!outfname) {
-		qflag = 1;
+	if (!outfname || outfname[0] == '|' || outfname[0] == '!')
 		setvbuf(outf, buf, _IOLBF, BUFSIZ);
-	}
-	else if (optind < argc)
+	if (outfname && optind < argc) {
 		interactive = 0;
-	else
 		qflag = 1;
+	}
 
-	for (c = 0, tcp = tcbtab; c < MAX_PROCS; c++, tcp++) {
+	for (c = 0; c < tcbtabsize; c++) {
+		tcp = tcbtab[c];
 		/* Reinitialize the output since it may have changed. */
 		tcp->outf = outf;
 		if (!(tcp->flags & TCB_INUSE) || !(tcp->flags & TCB_ATTACHED))
@@ -358,6 +400,68 @@ char *argv[];
 			continue;
 		}
 #else /* !USE_PROCFS */
+# ifdef LINUX
+		if (tcp->flags & TCB_CLONE_THREAD)
+			continue;
+		if (followfork) {
+			char procdir[MAXPATHLEN];
+			DIR *dir;
+			sprintf(procdir, "/proc/%d/task", tcp->pid);
+			dir = opendir(procdir);
+			if (dir != NULL) {
+				unsigned int ntid = 0, nerr = 0;
+				struct dirent *de;
+				int tid;
+				while ((de = readdir(dir)) != NULL) {
+					if (de->d_fileno == 0 ||
+					    de->d_name[0] == '.')
+						continue;
+					tid = atoi(de->d_name);
+					if (tid <= 0)
+						continue;
+					++ntid;
+					if (ptrace(PTRACE_ATTACH, tid,
+						   (char *) 1, 0) < 0)
+						++nerr;
+					else if (tid != tcbtab[c]->pid) {
+						if (nprocs == tcbtabsize &&
+						    expand_tcbtab())
+							tcp = NULL;
+						else
+							tcp = alloctcb(tid);
+						if (tcp == NULL) {
+							fprintf(stderr, "%s: out of memory\n",
+								progname);
+							exit(1);
+						}
+						tcp->flags |= TCB_ATTACHED|TCB_CLONE_THREAD|TCB_CLONE_DETACHED|TCB_FOLLOWFORK;
+						tcbtab[c]->nchildren++;
+						tcbtab[c]->nclone_threads++;
+						tcbtab[c]->nclone_detached++;
+						tcp->parent = tcbtab[c];
+					}
+				}
+				closedir(dir);
+				if (nerr == ntid) {
+					perror("attach: ptrace(PTRACE_ATTACH, ...)");
+					droptcb(tcp);
+					continue;
+				}
+				if (!qflag) {
+					ntid -= nerr;
+					if (ntid > 1)
+						fprintf(stderr, "\
+Process %u attached with %u threads - interrupt to quit\n",
+							tcp->pid, ntid);
+					else
+						fprintf(stderr, "\
+Process %u attached - interrupt to quit\n",
+							tcp->pid);
+				}
+				continue;
+			}
+		}
+# endif
 		if (ptrace(PTRACE_ATTACH, tcp->pid, (char *) 1, 0) < 0) {
 			perror("attach: ptrace(PTRACE_ATTACH, ...)");
 			droptcb(tcp);
@@ -367,17 +471,23 @@ char *argv[];
 		if (!qflag)
 			fprintf(stderr,
 				"Process %u attached - interrupt to quit\n",
-				pid);
+				tcp->pid);
 	}
 
-	if (optind < argc) {
+	if (!pflag_seen) {
 		struct stat statbuf;
 		char *filename;
 		char pathname[MAXPATHLEN];
 
 		filename = argv[optind];
-		if (strchr(filename, '/'))
+		if (strchr(filename, '/')) {
+			if (strlen(filename) > sizeof pathname - 1) {
+				errno = ENAMETOOLONG;
+				perror("strace: exec");
+				exit(1);
+			}
 			strcpy(pathname, filename);
+		}
 #ifdef USE_DEBUGGING_EXEC
 		/*
 		 * Debuggers customarily check the current directory
@@ -402,6 +512,8 @@ char *argv[];
 					getcwd(pathname, MAXPATHLEN);
 					len = strlen(pathname);
 				}
+				else if (n > sizeof pathname - 1)
+					continue;
 				else {
 					strncpy(pathname, path, n);
 					len = n;
@@ -409,7 +521,12 @@ char *argv[];
 				if (len && pathname[len - 1] != '/')
 					pathname[len++] = '/';
 				strcpy(pathname + len, filename);
-				if (stat(pathname, &statbuf) == 0)
+				if (stat(pathname, &statbuf) == 0 &&
+				    /* Accept only regular files
+				       with some execute bits set.
+				       XXX not perfect, might still fail */
+				    S_ISREG(statbuf.st_mode) &&
+				    (statbuf.st_mode & 0111))
 					break;
 			}
 		}
@@ -438,9 +555,9 @@ char *argv[];
 			pause();
 #else /* FREEBSD */
 			kill(getpid(), SIGSTOP); /* stop HERE */
-#endif /* FREEBSD */			
+#endif /* FREEBSD */
 #else /* !USE_PROCFS */
-			if (outf!=stderr)	
+			if (outf!=stderr)
 				close(fileno (outf));
 
 			if (ptrace(PTRACE_TRACEME, 0, (char *) 1, 0) < 0) {
@@ -501,13 +618,13 @@ char *argv[];
 			}
 #endif /* USE_PROCFS */
 #ifndef USE_PROCFS
-			fake_execve(tcp, pathname, &argv[optind], environ);
+			if (!cflag)
+				fake_execve(tcp, pathname,
+					    &argv[optind], environ);
 #endif /* !USE_PROCFS */
 			break;
 		}
 	}
-	else if (pflag_seen == 0)
-		usage(stderr, 1);
 
 	sigemptyset(&empty_set);
 	sigemptyset(&blocked_set);
@@ -536,6 +653,14 @@ char *argv[];
 #ifdef USE_PROCFS
 	sa.sa_handler = reaper;
 	sigaction(SIGCHLD, &sa, NULL);
+#else
+	/* Make sure SIGCHLD has the default action so that waitpid
+	   definitely works without losing track of children.  The user
+	   should not have given us a bogus state to inherit, but he might
+	   have.  Arguably we should detect SIG_IGN here and pass it on
+	   to children, but probably noone really needs that.  */
+	sa.sa_handler = SIG_DFL;
+	sigaction(SIGCHLD, &sa, NULL);
 #endif /* USE_PROCFS */
 
 	if (trace() < 0)
@@ -569,6 +694,33 @@ struct tcb *tcp;
 	return;
 }
 
+int
+expand_tcbtab()
+{
+	/* Allocate some more TCBs and expand the table.
+	   We don't want to relocate the TCBs because our
+	   callers have pointers and it would be a pain.
+	   So tcbtab is a table of pointers.  Since we never
+	   free the TCBs, we allocate a single chunk of many.  */
+	struct tcb **newtab = (struct tcb **)
+		realloc(tcbtab, 2 * tcbtabsize * sizeof tcbtab[0]);
+	struct tcb *newtcbs = (struct tcb *) calloc(tcbtabsize,
+						    sizeof *newtcbs);
+	int i;
+	if (newtab == NULL || newtcbs == NULL) {
+		if (newtab != NULL)
+			free(newtab);
+		return 1;
+	}
+	for (i = tcbtabsize; i < 2 * tcbtabsize; ++i)
+		newtab[i] = &newtcbs[i - tcbtabsize];
+	tcbtabsize *= 2;
+	tcbtab = newtab;
+
+	return 0;
+}
+
+
 struct tcb *
 alloctcb(pid)
 int pid;
@@ -576,11 +728,17 @@ int pid;
 	int i;
 	struct tcb *tcp;
 
-	for (i = 0, tcp = tcbtab; i < MAX_PROCS; i++, tcp++) {
+	for (i = 0; i < tcbtabsize; i++) {
+		tcp = tcbtab[i];
 		if ((tcp->flags & TCB_INUSE) == 0) {
 			tcp->pid = pid;
 			tcp->parent = NULL;
 			tcp->nchildren = 0;
+			tcp->nzombies = 0;
+#ifdef TCB_CLONE_THREAD
+			tcp->nclone_threads = tcp->nclone_detached = 0;
+			tcp->nclone_waiting = 0;
+#endif
 			tcp->flags = TCB_INUSE | TCB_STARTUP;
 			tcp->outf = outf; /* Initialise to current out file */
 			tcp->stime.tv_sec = 0;
@@ -602,7 +760,8 @@ int attaching;
 	char proc[32];
 	long arg;
 #ifdef SVR4
-	sysset_t sc_enter, sc_exit;
+	int i;
+	sysset_t syscalls;
 	sigset_t signals;
 	fltset_t faults;
 #endif
@@ -708,7 +867,7 @@ int attaching;
 		perror("strace: PIOCSTOP");
 		return -1;
 	}
-#endif	
+#endif
 #ifdef PIOCSET
 	/* Set Run-on-Last-Close. */
 	arg = PR_RLC;
@@ -723,7 +882,7 @@ int attaching;
 		return -1;
 	}
 #else  /* !PIOCSET */
-#ifndef FREEBSD	
+#ifndef FREEBSD
 	if (ioctl(tcp->pfd, PIOCSRLC) < 0) {
 		perror("PIOCSRLC");
 		return -1;
@@ -746,26 +905,53 @@ int attaching;
 #endif /* FREEBSD */
 #endif /* !PIOCSET */
 #ifndef FREEBSD
-	/* Enable all syscall entries. */
-	prfillset(&sc_enter);
-	if (IOCTL(tcp->pfd, PIOCSENTRY, &sc_enter) < 0) {
+	/* Enable all syscall entries we care about. */
+	premptyset(&syscalls);
+	for (i = 1; i < MAX_QUALS; ++i) {
+		if (i > (sizeof syscalls) * CHAR_BIT) break;
+		if (qual_flags [i] & QUAL_TRACE) praddset (&syscalls, i);
+	}
+	praddset (&syscalls, SYS_execve);
+	if (followfork) {
+		praddset (&syscalls, SYS_fork);
+#ifdef SYS_forkall
+		praddset (&syscalls, SYS_forkall);
+#endif
+#ifdef SYS_fork1
+		praddset (&syscalls, SYS_fork1);
+#endif
+#ifdef SYS_rfork1
+		praddset (&syscalls, SYS_rfork1);
+#endif
+#ifdef SYS_rforkall
+		praddset (&syscalls, SYS_rforkall);
+#endif
+	}
+	if (IOCTL(tcp->pfd, PIOCSENTRY, &syscalls) < 0) {
 		perror("PIOCSENTRY");
 		return -1;
 	}
-	/* Enable all syscall exits. */
-	prfillset(&sc_exit);
-	if (IOCTL(tcp->pfd, PIOCSEXIT, &sc_exit) < 0) {
+	/* Enable the syscall exits. */
+	if (IOCTL(tcp->pfd, PIOCSEXIT, &syscalls) < 0) {
 		perror("PIOSEXIT");
 		return -1;
 	}
-	/* Enable all signals. */
-	prfillset(&signals);
+	/* Enable signals we care about. */
+	premptyset(&signals);
+	for (i = 1; i < MAX_QUALS; ++i) {
+		if (i > (sizeof signals) * CHAR_BIT) break;
+		if (qual_flags [i] & QUAL_SIGNAL) praddset (&signals, i);
+	}
 	if (IOCTL(tcp->pfd, PIOCSTRACE, &signals) < 0) {
 		perror("PIOCSTRACE");
 		return -1;
 	}
-	/* Enable all faults. */
-	prfillset(&faults);
+	/* Enable faults we care about */
+	premptyset(&faults);
+	for (i = 1; i < MAX_QUALS; ++i) {
+		if (i > (sizeof faults) * CHAR_BIT) break;
+		if (qual_flags [i] & QUAL_FAULT) praddset (&faults, i);
+	}
 	if (IOCTL(tcp->pfd, PIOCSFAULT, &faults) < 0) {
 		perror("PIOCSFAULT");
 		return -1;
@@ -786,19 +972,19 @@ int attaching;
 		 */
 		kill(tcp->pid, SIGINT);
 #else /* !MIPS */
-#ifdef PRSABORT	
+#ifdef PRSABORT
 		/* The child is in a pause(), abort it. */
 		arg = PRSABORT;
 		if (IOCTL (tcp->pfd, PIOCRUN, &arg) < 0) {
 			perror("PIOCRUN");
 			return -1;
 		}
-#endif		
+#endif
 #endif /* !MIPS*/
 #ifdef FREEBSD
 		/* wake up the child if it received the SIGSTOP */
 		kill(tcp->pid, SIGCONT);
-#endif		
+#endif
 		for (;;) {
 			/* Wait for the child to do something. */
 			if (IOCTL_WSTOP (tcp) < 0) {
@@ -817,7 +1003,7 @@ int attaching;
 			if (IOCTL(tcp->pfd, PIOCRUN, &arg) < 0) {
 #else /* FREEBSD */
 			if (IOCTL(tcp->pfd, PIOCRUN, 0) < 0) {
-#endif /* FREEBSD */			  
+#endif /* FREEBSD */
 				perror("PIOCRUN");
 				return -1;
 			}
@@ -827,13 +1013,13 @@ int attaching;
 			if (tcp->status.PR_WHY == PR_SIGNALLED &&
 			    tcp->status.PR_WHAT == SIGSTOP)
 			        kill(tcp->pid, SIGCONT);
-#endif			
+#endif
 		}
 #ifndef FREEBSD
 	}
 #else /* FREEBSD */
 	} else {
-		if (attaching < 2) { 
+		if (attaching < 2) {
 			/* We are attaching to an already running process.
 			 * Try to figure out the state of the process in syscalls,
 			 * to handle the first event well.
@@ -841,7 +1027,7 @@ int attaching;
 			 * process, which tells where it is stopped (if it is). */
 			FILE * status;
 			char wchan[20]; /* should be enough */
-			
+
 			sprintf(proc, "/proc/%d/status", tcp->pid);
 			status = fopen(proc, "r");
 			if (status &&
@@ -875,14 +1061,15 @@ int attaching;
 
 #endif /* USE_PROCFS */
 
-static struct tcb *
+struct tcb *
 pid2tcb(pid)
 int pid;
 {
 	int i;
 	struct tcb *tcp;
 
-	for (i = 0, tcp = tcbtab; i < MAX_PROCS; i++, tcp++) {
+	for (i = 0; i < tcbtabsize; i++) {
+		tcp = tcbtab[i];
 		if (pid && tcp->pid != pid)
 			continue;
 		if (tcp->flags & TCB_INUSE)
@@ -898,9 +1085,9 @@ pfd2tcb(pfd)
 int pfd;
 {
 	int i;
-	struct tcb *tcp;
 
-	for (i = 0, tcp = tcbtab; i < MAX_PROCS; i++, tcp++) {
+	for (i = 0; i < tcbtabsize; i++) {
+		struct tcb *tcp = tcbtab[i];
 		if (tcp->pfd != pfd)
 			continue;
 		if (tcp->flags & TCB_INUSE)
@@ -917,8 +1104,34 @@ struct tcb *tcp;
 {
 	if (tcp->pid == 0)
 		return;
+#ifdef TCB_CLONE_THREAD
+	if (tcp->nclone_threads > 0) {
+		/* There are other threads left in this process, but this
+		   is the one whose PID represents the whole process.
+		   We need to keep this record around as a zombie until
+		   all the threads die.  */
+		tcp->flags |= TCB_EXITING;
+		return;
+	}
+#endif
 	nprocs--;
 	tcp->pid = 0;
+
+	if (tcp->parent != NULL) {
+		tcp->parent->nchildren--;
+#ifdef TCB_CLONE_THREAD
+		if (tcp->flags & TCB_CLONE_DETACHED)
+			tcp->parent->nclone_detached--;
+		if (tcp->flags & TCB_CLONE_THREAD)
+			tcp->parent->nclone_threads--;
+#endif
+#ifdef TCB_CLONE_DETACHED
+		if (!(tcp->flags & TCB_CLONE_DETACHED))
+#endif
+			tcp->parent->nzombies++;
+		tcp->parent = NULL;
+	}
+
 	tcp->flags = 0;
 	if (tcp->pfd != -1) {
 		close(tcp->pfd);
@@ -932,19 +1145,15 @@ struct tcb *tcp;
 			close(tcp->pfd_status);
 			tcp->pfd_status = -1;
 		}
-#endif /* !FREEBSD */		
+#endif /* !FREEBSD */
 #ifdef USE_PROCFS
-		rebuild_pollv();
+		rebuild_pollv(); /* Note, flags needs to be cleared by now.  */
 #endif
 	}
-	if (tcp->parent != NULL) {
-		tcp->parent->nchildren--;
-		tcp->parent = NULL;
-	}
-#if 0
-	if (tcp->outf != stderr)
+
+	if (outfname && followfork > 1 && tcp->outf)
 		fclose(tcp->outf);
-#endif
+
 	tcp->outf = 0;
 }
 
@@ -962,6 +1171,10 @@ struct tcb *tcp;
 		return -1;
 	}
 	tcp->flags &= ~TCB_SUSPENDED;
+#ifdef TCB_CLONE_THREAD
+	if (tcp->flags & TCB_CLONE_THREAD)
+		tcp->parent->nclone_waiting--;
+#endif
 
 	if (ptrace(PTRACE_SYSCALL, tcp->pid, (char *) 1, 0) < 0) {
 		perror("resume: ptrace(PTRACE_SYSCALL, ...)");
@@ -984,7 +1197,7 @@ int sig;
 {
 	int error = 0;
 #ifdef LINUX
-	int status;
+	int status, resumed;
 #endif
 
 	if (tcp->flags & TCB_BPTSET)
@@ -1017,11 +1230,34 @@ int sig;
 	}
 	else {
 		for (;;) {
-			if (waitpid(tcp->pid, &status, 0) < 0) {
-				if (errno != ECHILD)
+#ifdef __WALL
+			if (wait4(tcp->pid, &status, __WALL, NULL) < 0) {
+				if (errno == ECHILD) /* Already gone.  */
+					break;
+				if (errno != EINVAL) {
 					perror("detach: waiting");
-				break;
+					break;
+				}
+#endif /* __WALL */
+				/* No __WALL here.  */
+				if (waitpid(tcp->pid, &status, 0) < 0) {
+					if (errno != ECHILD) {
+						perror("detach: waiting");
+						break;
+					}
+#ifdef __WCLONE
+					/* If no processes, try clones.  */
+					if (wait4(tcp->pid, &status, __WCLONE,
+						  NULL) < 0) {
+						if (errno != ECHILD)
+							perror("detach: waiting");
+						break;
+					}
+#endif /* __WCLONE */
+				}
+#ifdef __WALL
 			}
+#endif
 			if (!WIFSTOPPED(status)) {
 				/* Au revoir, mon ami. */
 				break;
@@ -1056,8 +1292,64 @@ int sig;
 #endif /* SUNOS4 */
 
 #ifndef USE_PROCFS
-	if (waiting_parent(tcp))
-		error = resume(tcp->parent);
+	resumed = 0;
+
+	/* XXX This won't always be quite right (but it never was).
+	   A waiter with argument 0 or < -1 is waiting for any pid in
+	   a particular pgrp, which this child might or might not be
+	   in.  The waiter will only wake up if it's argument is -1
+	   or if it's waiting for tcp->pid's pgrp.  It makes a
+	   difference to wake up a waiter when there might be more
+	   traced children, because it could get a false ECHILD
+	   error.  OTOH, if this was the last child in the pgrp, then
+	   it ought to wake up and get ECHILD.  We would have to
+	   search the system for all pid's in the pgrp to be sure.
+
+	     && (t->waitpid == -1 ||
+		 (t->waitpid == 0 && getpgid (tcp->pid) == getpgid (t->pid))
+		 || (t->waitpid < 0 && t->waitpid == -getpid (t->pid)))
+	*/
+
+	if (tcp->parent &&
+	    (tcp->parent->flags & TCB_SUSPENDED) &&
+	    (tcp->parent->waitpid <= 0 || tcp->parent->waitpid == tcp->pid)) {
+ 		error = resume(tcp->parent);
+		++resumed;
+	}
+#ifdef TCB_CLONE_THREAD
+	if (tcp->parent && tcp->parent->nclone_waiting > 0) {
+		/* Some other threads of our parent are waiting too.  */
+		unsigned int i;
+
+		/* Resume all the threads that were waiting for this PID.  */
+		for (i = 0; i < tcbtabsize; i++) {
+			struct tcb *t = tcbtab[i];
+			if (t->parent == tcp->parent && t != tcp
+			    && ((t->flags & (TCB_CLONE_THREAD|TCB_SUSPENDED))
+				== (TCB_CLONE_THREAD|TCB_SUSPENDED))
+			    && t->waitpid == tcp->pid) {
+				error |= resume (t);
+				++resumed;
+			}
+		}
+		if (resumed == 0)
+			/* Noone was waiting for this PID in particular,
+			   so now we might need to resume some wildcarders.  */
+			for (i = 0; i < tcbtabsize; i++) {
+				struct tcb *t = tcbtab[i];
+				if (t->parent == tcp->parent && t != tcp
+				    && ((t->flags
+					 & (TCB_CLONE_THREAD|TCB_SUSPENDED))
+					== (TCB_CLONE_THREAD|TCB_SUSPENDED))
+				    && t->waitpid <= 0
+					) {
+					error |= resume (t);
+					break;
+				}
+			}
+	}
+#endif
+
 #endif /* !USE_PROCFS */
 
 	if (!qflag)
@@ -1095,7 +1387,8 @@ cleanup()
 	int i;
 	struct tcb *tcp;
 
-	for (i = 0, tcp = tcbtab; i < MAX_PROCS; i++, tcp++) {
+	for (i = 0; i < tcbtabsize; i++) {
+		tcp = tcbtab[i];
 		if (!(tcp->flags & TCB_INUSE))
 			continue;
 		if (debug)
@@ -1126,10 +1419,10 @@ int sig;
 
 #ifndef HAVE_STRERROR
 
-#ifndef SYS_ERRLIST_DECLARED
+#if !HAVE_DECL_SYS_ERRLIST
 extern int sys_nerr;
 extern char *sys_errlist[];
-#endif /* SYS_ERRLIST_DECLARED */
+#endif /* HAVE_DECL_SYS_ERRLIST */
 
 const char *
 strerror(errno)
@@ -1148,13 +1441,12 @@ int errno;
 
 #ifndef HAVE_STRSIGNAL
 
-#ifndef SYS_SIGLIST_DECLARED
-#ifdef HAVE__SYS_SIGLIST
-	extern char *_sys_siglist[];
-#else
-	extern char *sys_siglist[];
+#if defined HAVE_SYS_SIGLIST && !defined HAVE_DECL_SYS_SIGLIST
+extern char *sys_siglist[];
+#endif
+#if defined HAVE_SYS__SIGLIST && !defined HAVE_DECL__SYS_SIGLIST
+extern char *_sys_siglist[];
 #endif
-#endif /* SYS_SIGLIST_DECLARED */
 
 const char *
 strsignal(sig)
@@ -1181,9 +1473,17 @@ static void
 rebuild_pollv()
 {
 	int i, j;
-	struct tcb *tcp;
 
-	for (i = j = 0, tcp = tcbtab; i < MAX_PROCS; i++, tcp++) {
+	if (pollv != NULL)
+		free (pollv);
+	pollv = (struct pollfd *) malloc(nprocs * sizeof pollv[0]);
+	if (pollv == NULL) {
+		fprintf(stderr, "strace: out of memory for poll vector\n");
+		exit(1);
+	}
+
+	for (i = j = 0; i < tcbtabsize; i++) {
+		struct tcb *tcp = tcbtab[i];
 		if (!(tcp->flags & TCB_INUSE))
 			continue;
 		pollv[j].fd = tcp->pfd;
@@ -1378,7 +1678,7 @@ static int
 trace()
 {
 #ifdef POLL_HACK
-	struct tcb *in_syscall;
+	struct tcb *in_syscall = NULL;
 #endif
 	struct tcb *tcp;
 	int pfd;
@@ -1415,7 +1715,7 @@ trace()
 		        /* On some systems (e.g. UnixWare) we get too much ugly
 			   "unfinished..." stuff when multiple proceses are in
 			   syscalls.  Here's a nasty hack */
-		    
+
 			if (in_syscall) {
 				struct pollfd pv;
 				tcp = in_syscall;
@@ -1456,7 +1756,9 @@ trace()
 			fprintf(stderr, "unknown pfd: %u\n", pfd);
 			exit(1);
 		}
+#ifdef POLL_HACK
 	FOUND:
+#endif
 		/* Get the status of the process. */
 		if (!interrupted) {
 #ifndef FREEBSD
@@ -1469,7 +1771,7 @@ trace()
 				ioctl_result = IOCTL_STATUS (tcp);
 			else
 			  	ioctl_result = IOCTL_WSTOP (tcp);
-#endif /* FREEBSD */			  
+#endif /* FREEBSD */
 			ioctl_errno = errno;
 #ifndef HAVE_POLLABLE_PROCFS
 			if (proc_poll_pipe[0] != -1) {
@@ -1494,7 +1796,7 @@ trace()
 				continue;
 #ifdef FREEBSD
 			case ENOTTY:
-#endif			  
+#endif
 			case ENOENT:
 				droptcb(tcp);
 				continue;
@@ -1510,8 +1812,8 @@ trace()
 			IOCTL (tcp->pfd, PIOCRUN, 0);
 			continue;
 		}
-#endif		
-		
+#endif
+
 		/* clear the just started flag */
 		tcp->flags &= ~TCB_STARTUP;
 
@@ -1531,7 +1833,7 @@ trace()
 				       &stime.tv_sec, &stime.tv_usec);
 			} else
 				stime.tv_sec = stime.tv_usec = 0;
-#else /* !FREEBSD */			
+#else /* !FREEBSD */
 			stime.tv_sec = tcp->status.pr_stime.tv_sec;
 			stime.tv_usec = tcp->status.pr_stime.tv_nsec/1000;
 #endif /* !FREEBSD */
@@ -1567,6 +1869,14 @@ trace()
 				tprintf("--- %s (%s) ---",
 					signame(what), strsignal(what));
 				printtrailer(tcp);
+#ifdef PR_INFO
+				if (tcp->status.PR_INFO.si_signo == what) {
+					printleader(tcp);
+					tprintf("    siginfo=");
+					printsiginfo(&tcp->status.PR_INFO, 1);
+					printtrailer(tcp);
+				}
+#endif
 			}
 			break;
 		case PR_FAULTED:
@@ -1579,18 +1889,18 @@ trace()
 #ifdef FREEBSD
 		case 0: /* handle case we polled for nothing */
 		  	continue;
-#endif			
+#endif
 		default:
 			fprintf(stderr, "odd stop %d\n", tcp->status.PR_WHY);
 			exit(1);
 			break;
 		}
 		arg = 0;
-#ifndef FREEBSD		
+#ifndef FREEBSD
 		if (IOCTL (tcp->pfd, PIOCRUN, &arg) < 0) {
-#else		  
+#else
 		if (IOCTL (tcp->pfd, PIOCRUN, 0) < 0) {
-#endif		  
+#endif
 			perror("PIOCRUN");
 			exit(1);
 		}
@@ -1600,6 +1910,79 @@ trace()
 
 #else /* !USE_PROCFS */
 
+#ifdef TCB_GROUP_EXITING
+/* Handle an exit detach or death signal that is taking all the
+   related clone threads with it.  This is called in three circumstances:
+   SIG == -1	TCP has already died (TCB_ATTACHED is clear, strace is parent).
+   SIG == 0	Continuing TCP will perform an exit_group syscall.
+   SIG == other	Continuing TCP with SIG will kill the process.
+*/
+static int
+handle_group_exit(struct tcb *tcp, int sig)
+{
+	/* We need to locate our records of all the clone threads
+	   related to TCP, either its children or siblings.  */
+	struct tcb *leader = ((tcp->flags & TCB_CLONE_THREAD)
+			      ? tcp->parent
+			      : tcp->nclone_detached > 0
+			      ? tcp : NULL);
+
+	if (sig < 0) {
+		if (leader != NULL && leader != tcp &&
+		    !(leader->flags & TCB_GROUP_EXITING))
+			fprintf(stderr,
+				"PANIC: handle_group_exit: %d leader %d\n",
+				tcp->pid, leader ? leader->pid : -1);
+		droptcb(tcp);	/* Already died.  */
+	}
+	else {
+		if (tcp->flags & TCB_ATTACHED) {
+		  	if (leader != NULL && leader != tcp) {
+				if (leader->flags & TCB_ATTACHED) {
+					/* We need to detach the leader so
+					   that the process death will be
+					   reported to its real parent.
+					   But we kill it first to prevent
+					   it doing anything before we kill
+					   the whole process in a moment.
+					   We can use PTRACE_KILL on a
+					   thread that's not already
+					   stopped.  Then the value we pass
+					   in PTRACE_DETACH just sets the
+					   death signal reported to the
+					   real parent.  */
+					ptrace(PTRACE_KILL, leader->pid, 0, 0);
+					if (debug)
+						fprintf(stderr,
+							" [%d exit %d kills %d]\n",
+							tcp->pid, sig, leader->pid);
+					detach(leader, sig);
+				}
+				else
+					leader->flags |= TCB_GROUP_EXITING;
+			}
+			detach(tcp, sig);
+		}
+		else if (ptrace(PTRACE_CONT, tcp->pid, (char *) 1, sig) < 0) {
+			perror("strace: ptrace(PTRACE_CONT, ...)");
+			cleanup();
+			return -1;
+		}
+		else {
+			if (leader != NULL)
+				leader->flags |= TCB_GROUP_EXITING;
+			if (leader != NULL && leader != tcp)
+				droptcb(tcp);
+			/* The leader will report to us as parent now,
+			   and then we'll get to the SIG==-1 case.  */
+			return 0;
+		}
+	}
+
+	return 0;
+}
+#endif
+
 static int
 trace()
 {
@@ -1620,14 +2003,14 @@ trace()
 #ifdef LINUX
 #ifdef __WALL
 		pid = wait4(-1, &status, wait4_options, cflag ? &ru : NULL);
-		if ((wait4_options & __WALL) && errno == EINVAL) {
+		if (pid < 0 && (wait4_options & __WALL) && errno == EINVAL) {
 			/* this kernel does not support __WALL */
 			wait4_options &= ~__WALL;
 			errno = 0;
 			pid = wait4(-1, &status, wait4_options,
 					cflag ? &ru : NULL);
 		}
-		if (!(wait4_options & __WALL) && errno == ECHILD) {
+		if (pid < 0 && !(wait4_options & __WALL) && errno == ECHILD) {
 			/* most likely a "cloned" process */
 			pid = wait4(-1, &status, __WCLONE,
 					cflag ? &ru : NULL);
@@ -1679,25 +2062,39 @@ trace()
 
 		/* Look up `pid' in our table. */
 		if ((tcp = pid2tcb(pid)) == NULL) {
-#if 0 /* XXX davidm */ /* WTA: disabled again */
-			struct tcb *tcpchild;
-
-			if ((tcpchild = alloctcb(pid)) == NULL) {
-				fprintf(stderr, " [tcb table full]\n");
-				kill(pid, SIGKILL); /* XXX */
-				return 0;
+#ifdef LINUX
+			if (followfork || followvfork) {
+				/* This is needed to go with the CLONE_PTRACE
+				   changes in process.c/util.c: we might see
+				   the child's initial trap before we see the
+				   parent return from the clone syscall.
+				   Leave the child suspended until the parent
+				   returns from its system call.  Only then
+				   will we have the association of parent and
+				   child so that we know how to do clearbpt
+				   in the child.  */
+				if ((tcp = alloctcb(pid)) == NULL) {
+					fprintf(stderr, " [tcb table full]\n");
+					kill(pid, SIGKILL); /* XXX */
+					return 0;
+				}
+				tcp->flags |= TCB_ATTACHED | TCB_SUSPENDED;
+				newoutf(tcp);
+				if (!qflag)
+					fprintf(stderr, "\
+Process %d attached (waiting for parent)\n",
+						pid);
 			}
-			tcpchild->flags |= TCB_ATTACHED;
-			newoutf(tcpchild);
-			tcp->nchildren++;
-			if (!qflag)
-				fprintf(stderr, "Process %d attached\n", pid);
-#else
-			fprintf(stderr, "unknown pid: %u\n", pid);
-			if (WIFSTOPPED(status))
-				ptrace(PTRACE_CONT, pid, (char *) 1, 0);
-			exit(1);
+			else
+				/* This can happen if a clone call used
+				   CLONE_PTRACE itself.  */
 #endif
+			{
+				fprintf(stderr, "unknown pid: %u\n", pid);
+				if (WIFSTOPPED(status))
+					ptrace(PTRACE_CONT, pid, (char *) 1, 0);
+				exit(1);
+			}
 		}
 		/* set current output file */
 		outf = tcp->outf;
@@ -1723,21 +2120,45 @@ trace()
 			if (!cflag
 			    && (qual_flags[WTERMSIG(status)] & QUAL_SIGNAL)) {
 				printleader(tcp);
-				tprintf("+++ killed by %s +++",
-					signame(WTERMSIG(status)));
+				tprintf("+++ killed by %s %s+++",
+					signame(WTERMSIG(status)),
+#ifdef WCOREDUMP
+					WCOREDUMP(status) ? "(core dumped) " :
+#endif
+					"");
 				printtrailer(tcp);
 			}
+#ifdef TCB_GROUP_EXITING
+			handle_group_exit(tcp, -1);
+#else
 			droptcb(tcp);
+#endif
 			continue;
 		}
 		if (WIFEXITED(status)) {
 			if (debug)
 				fprintf(stderr, "pid %u exited\n", pid);
-			if (tcp->flags & TCB_ATTACHED)
+			if ((tcp->flags & TCB_ATTACHED)
+#ifdef TCB_GROUP_EXITING
+			    && !(tcp->parent && (tcp->parent->flags &
+						 TCB_GROUP_EXITING))
+#endif
+				)
 				fprintf(stderr,
 					"PANIC: attached pid %u exited\n",
 					pid);
+			if (tcp == tcp_last) {
+				if ((tcp->flags & (TCB_INSYSCALL|TCB_REPRINT))
+				    == TCB_INSYSCALL)
+					tprintf(" <unfinished ... exit status %d>\n",
+						WEXITSTATUS(status));
+				tcp_last = NULL;
+			}
+#ifdef TCB_GROUP_EXITING
+			handle_group_exit(tcp, -1);
+#else
 			droptcb(tcp);
+#endif
 			continue;
 		}
 		if (!WIFSTOPPED(status)) {
@@ -1805,15 +2226,33 @@ trace()
 			}
 			if (!cflag
 			    && (qual_flags[WSTOPSIG(status)] & QUAL_SIGNAL)) {
+				unsigned long addr = 0, pc = 0;
+#ifdef PT_GETSIGINFO
+#				define PSR_RI	41
+				struct siginfo si;
+				unsigned long psr;
+
+				upeek(pid, PT_CR_IPSR, &psr);
+				upeek(pid, PT_CR_IIP, &pc);
+
+				pc += (psr >> PSR_RI) & 0x3;
+				ptrace(PT_GETSIGINFO, pid, 0, (long) &si);
+				addr = (unsigned long) si.si_addr;
+#endif
 				printleader(tcp);
-				tprintf("--- %s (%s) ---",
+				tprintf("--- %s (%s) @ %lx (%lx) ---",
 					signame(WSTOPSIG(status)),
-					strsignal(WSTOPSIG(status)));
+					strsignal(WSTOPSIG(status)), pc, addr);
 				printtrailer(tcp);
 			}
-			if ((tcp->flags & TCB_ATTACHED) &&
+			if (((tcp->flags & TCB_ATTACHED) ||
+			     tcp->nclone_threads > 0) &&
 				!sigishandled(tcp, WSTOPSIG(status))) {
+#ifdef TCB_GROUP_EXITING
+				handle_group_exit(tcp, WSTOPSIG(status));
+#else
 				detach(tcp, WSTOPSIG(status));
+#endif
 				continue;
 			}
 			if (ptrace(PTRACE_SYSCALL, pid, (char *) 1,
@@ -1836,6 +2275,13 @@ trace()
 			continue;
 		}
 		if (tcp->flags & TCB_EXITING) {
+#ifdef TCB_GROUP_EXITING
+			if (tcp->flags & TCB_GROUP_EXITING) {
+				if (handle_group_exit(tcp, 0) < 0)
+					return -1;
+				continue;
+			}
+#endif
 			if (tcp->flags & TCB_ATTACHED)
 				detach(tcp, 0);
 			else if (ptrace(PTRACE_CONT, pid, (char *) 1, 0) < 0) {
@@ -1884,8 +2330,14 @@ va_dcl
 	va_list args;
 
 	VA_START(args, fmt);
-	if (outf)
-		curcol += vfprintf(outf, fmt, args);
+	if (outf) {
+		int n = vfprintf(outf, fmt, args);
+		if (n < 0 && outf != stderr)
+			perror(outfname == NULL
+			       ? "<writing to pipe>" : outfname);
+		else
+			curcol += n;
+	}
 	va_end(args);
 	return;
 }
@@ -1956,7 +2408,7 @@ int mp_ioctl (int fd, int cmd, void *arg, int size) {
 
 	struct iovec iov[2];
 	int n = 1;
-	
+
 	iov[0].iov_base = &cmd;
 	iov[0].iov_len = sizeof cmd;
 	if (arg) {
@@ -1964,7 +2416,7 @@ int mp_ioctl (int fd, int cmd, void *arg, int size) {
 		iov[1].iov_base = arg;
 		iov[1].iov_len = size;
 	}
-	
+
 	return writev (fd, iov, n);
 }
 
