@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: system.c,v 1.24 2001/04/18 15:11:52 hughesj Exp $
+ *	$Id: system.c,v 1.32 2004/09/04 03:39:20 roland Exp $
  */
 
 #include "defs.h"
@@ -49,17 +49,6 @@
 
 #include <sys/syscall.h>
 
-#ifdef SYS_personality
-/* Workaround for kernel namespace pollution. */
-#define _LINUX_PTRACE_H
-/* Yuck yuck yuck.  We can't include linux/ptrace.h, but personality.h
-   makes a declaration with struct pt_regs, which is defined there. */
-struct pt_regs;
-#define sys_personality kernel_sys_personality
-#include <linux/personality.h>
-#undef sys_personality
-#endif /* SYS_personality */
-
 #ifdef SYS_capget
 #include <linux/capability.h>
 #endif
@@ -72,13 +61,13 @@ struct pt_regs;
 #include <linux/utsname.h>
 #endif
 
-#ifdef HAVE_ASM_SYSMIPS_H
+#ifdef MIPS
 #include <asm/sysmips.h>
 #endif
 
 #include <linux/sysctl.h>
 
-static struct xlat mount_flags[] = {
+static const struct xlat mount_flags[] = {
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_NOSUID,	"MS_NOSUID"	},
 	{ MS_NODEV,	"MS_NODEV"	},
@@ -124,34 +113,30 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat personality_options[] = {
-#ifdef PER_LINUX
-	{ PER_LINUX,	"PER_LINUX"	},
-#endif
-#ifdef PER_LINUX_32BIT
-	{ PER_LINUX_32BIT,	"PER_LINUX"	},
-#endif
-#ifdef PER_SVR4
-	{ PER_SVR4,	"PER_SVR4"	},
-#endif
-#ifdef PER_SVR3
-	{ PER_SVR3,	"PER_SVR3"	},
-#endif
-#ifdef PER_SCOSVR3
-	{ PER_SCOSVR3,	"PER_SCOSVR3"	},
-#endif
-#ifdef PER_WYSEV386
-	{ PER_WYSEV386,	"PER_WYSEV386"	},
-#endif
-#ifdef PER_ISCR4
-	{ PER_ISCR4,	"PER_ISCR4"	},
-#endif
-#ifdef PER_BSD
-	{ PER_BSD,	"PER_BSD"	},
-#endif
-#ifdef PER_XENIX
-	{ PER_XENIX,	"PER_XENIX"	},
-#endif
+/* These are not macros, but enums.  We just copy the values by hand
+   from Linux 2.6.9 here.  */
+static const struct xlat personality_options[] = {
+	{ 0,		"PER_LINUX"	},
+	{ 0x00800000,	"PER_LINUX_32BIT"},
+	{ 0x04100001,	"PER_SVR4"	},
+	{ 0x05000002,	"PER_SVR3"	},
+	{ 0x07000003,	"PER_SCOSVR3"	},
+	{ 0x06000003,	"PER_OSR5"	},
+	{ 0x05000004,	"PER_WYSEV386"	},
+	{ 0x04000005,	"PER_ISCR4"	},
+	{ 0x00000006,	"PER_BSD"	},
+	{ 0x04000006,	"PER_SUNOS"	},
+	{ 0x05000007,	"PER_XENIX"	},
+	{ 0x00000008,	"PER_LINUX32"	},
+	{ 0x08000008,	"PER_LINUX32_3GB"},
+	{ 0x04000009,	"PER_IRIX32"	},
+	{ 0x0400000a,	"PER_IRIXN32"	},
+	{ 0x0400000b,	"PER_IRIX64"	},
+	{ 0x0000000c,	"PER_RISCOS"	},
+	{ 0x0400000d,	"PER_SOLARIS"	},
+	{ 0x0410000e,	"PER_UW7"	},
+	{ 0x0000000f,	"PER_OSF4"	},
+	{ 0x00000010,	"PER_HPUX"	},
 	{ 0,		NULL		},
 };
 
@@ -165,19 +150,19 @@ struct tcb *tcp;
 }
 
 #include <linux/reboot.h>
-static struct xlat bootflags1[] = {
+static const struct xlat bootflags1[] = {
 	{ LINUX_REBOOT_MAGIC1,	"LINUX_REBOOT_MAGIC1"	},
 	{ 0,			NULL			},
 };
 
-static struct xlat bootflags2[] = {
+static const struct xlat bootflags2[] = {
 	{ LINUX_REBOOT_MAGIC2,	"LINUX_REBOOT_MAGIC2"	},
 	{ LINUX_REBOOT_MAGIC2A,	"LINUX_REBOOT_MAGIC2A"	},
 	{ LINUX_REBOOT_MAGIC2B,	"LINUX_REBOOT_MAGIC2B"	},
 	{ 0,			NULL			},
 };
 
-static struct xlat bootflags3[] = {
+static const struct xlat bootflags3[] = {
 	{ LINUX_REBOOT_CMD_CAD_OFF,	"LINUX_REBOOT_CMD_CAD_OFF"	},
 	{ LINUX_REBOOT_CMD_RESTART,	"LINUX_REBOOT_CMD_RESTART"	},
 	{ LINUX_REBOOT_CMD_HALT,	"LINUX_REBOOT_CMD_HALT"		},
@@ -209,7 +194,7 @@ struct tcb *tcp;
 }
 
 #ifdef M68K
-static struct xlat cacheflush_scope[] = {
+static const struct xlat cacheflush_scope[] = {
 #ifdef FLUSH_SCOPE_LINE
 	{ FLUSH_SCOPE_LINE,	"FLUSH_SCOPE_LINE" },
 #endif
@@ -222,7 +207,7 @@ static struct xlat cacheflush_scope[] = {
 	{ 0,			NULL },
 };
 
-static struct xlat cacheflush_flags[] = {
+static const struct xlat cacheflush_flags[] = {
 #ifdef FLUSH_CACHE_BOTH
 	{ FLUSH_CACHE_BOTH,	"FLUSH_CACHE_BOTH" },
 #endif
@@ -277,7 +262,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat bootflags[] = {
+static const struct xlat bootflags[] = {
 	{ RB_AUTOBOOT,	"RB_AUTOBOOT"	},	/* for system auto-booting itself */
 	{ RB_ASKNAME,	"RB_ASKNAME"	},	/* ask for file name to reboot from */
 	{ RB_SINGLE,	"RB_SINGLE"	},	/* reboot to single user only */
@@ -336,7 +321,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat mountflags[] = {
+static const struct xlat mountflags[] = {
 	{ M_RDONLY,	"M_RDONLY"	},
 	{ M_NOSUID,	"M_NOSUID"	},
 	{ M_NEWTYPE,	"M_NEWTYPE"	},
@@ -356,7 +341,7 @@ static struct xlat mountflags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat nfsflags[] = {
+static const struct xlat nfsflags[] = {
 	{ NFSMNT_SOFT,		"NFSMNT_SOFT"		},
 	{ NFSMNT_WSIZE,		"NFSMNT_WSIZE"		},
 	{ NFSMNT_RSIZE,		"NFSMNT_RSIZE"		},
@@ -477,7 +462,7 @@ struct tcb *tcp;
 	return printargs(tcp);
 }
 
-static struct xlat ex_auth_flags[] = {
+static const struct xlat ex_auth_flags[] = {
 	{ AUTH_UNIX,	"AUTH_UNIX"	},
 	{ AUTH_DES,	"AUTH_DES"	},
 	{ 0,		NULL		},
@@ -523,7 +508,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat sysconflimits[] = {
+static const struct xlat sysconflimits[] = {
 #ifdef	_SC_ARG_MAX
 	{ _SC_ARG_MAX,	"_SC_ARG_MAX"	},	/* space for argv & envp */
 #endif
@@ -564,7 +549,7 @@ struct tcb *tcp;
 #endif /* SUNOS4 */
 
 #if defined(SUNOS4) || defined(FREEBSD)
-static struct xlat pathconflimits[] = {
+static const struct xlat pathconflimits[] = {
 #ifdef	_PC_LINK_MAX
 	{ _PC_LINK_MAX,	"_PC_LINK_MAX"	},	/* max links to file/dir */
 #endif
@@ -631,7 +616,7 @@ struct tcb *tcp;
 #include <sys/systeminfo.h>
 #include <sys/utsname.h>
 
-static struct xlat sysconfig_options[] = {
+static const struct xlat sysconfig_options[] = {
 #ifdef _CONFIG_NGROUPS
 	{ _CONFIG_NGROUPS,		"_CONFIG_NGROUPS"		},
 #endif
@@ -719,7 +704,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat sysinfo_options[] = {
+static const struct xlat sysinfo_options[] = {
 	{ SI_SYSNAME,		"SI_SYSNAME"		},
 	{ SI_HOSTNAME,		"SI_HOSTNAME"		},
 	{ SI_RELEASE,		"SI_RELEASE"		},
@@ -767,7 +752,7 @@ struct tcb *tcp;
 
 #include <sys/syssgi.h>
 
-static struct xlat syssgi_options[] = {
+static const struct xlat syssgi_options[] = {
 	{ SGI_SYSID,		"SGI_SYSID"		},
 #ifdef SGI_RDUBLK
 	{ SGI_RDUBLK,		"SGI_RDUBLK"		},
@@ -883,247 +868,247 @@ static struct xlat syssgi_options[] = {
 #ifdef SGI_GET_FP_PRECISE
 	{ SGI_GET_FP_PRECISE,	"SGI_GET_FP_PRECISE"	},
 #endif
-#ifdef SGI_GET_CONFIG_SMM	
+#ifdef SGI_GET_CONFIG_SMM
 	{ SGI_GET_CONFIG_SMM,	"SGI_GET_CONFIG_SMM"	},
 #endif
-#ifdef SGI_FP_IMPRECISE_SUPP	
+#ifdef SGI_FP_IMPRECISE_SUPP
 	{ SGI_FP_IMPRECISE_SUPP,"SGI_FP_IMPRECISE_SUPP"	},
 #endif
-#ifdef SGI_CONFIG_NSMM_SUPP	
+#ifdef SGI_CONFIG_NSMM_SUPP
 	{ SGI_CONFIG_NSMM_SUPP,	"SGI_CONFIG_NSMM_SUPP"	},
 #endif
-#ifdef SGI_RT_TSTAMP_CREATE    
+#ifdef SGI_RT_TSTAMP_CREATE
 	{ SGI_RT_TSTAMP_CREATE,	"SGI_RT_TSTAMP_CREATE"	},
 #endif
-#ifdef SGI_RT_TSTAMP_DELETE    
+#ifdef SGI_RT_TSTAMP_DELETE
 	{ SGI_RT_TSTAMP_DELETE,	"SGI_RT_TSTAMP_DELETE"	},
 #endif
-#ifdef SGI_RT_TSTAMP_START     
+#ifdef SGI_RT_TSTAMP_START
 	{ SGI_RT_TSTAMP_START,	"SGI_RT_TSTAMP_START"	},
 #endif
-#ifdef SGI_RT_TSTAMP_STOP      
+#ifdef SGI_RT_TSTAMP_STOP
 	{ SGI_RT_TSTAMP_STOP,	"SGI_RT_TSTAMP_STOP"	},
 #endif
-#ifdef SGI_RT_TSTAMP_ADDR      
+#ifdef SGI_RT_TSTAMP_ADDR
 	{ SGI_RT_TSTAMP_ADDR,	"SGI_RT_TSTAMP_ADDR"	},
 #endif
-#ifdef SGI_RT_TSTAMP_MASK      
+#ifdef SGI_RT_TSTAMP_MASK
 	{ SGI_RT_TSTAMP_MASK,	"SGI_RT_TSTAMP_MASK"	},
 #endif
-#ifdef SGI_RT_TSTAMP_EOB_MODE  
+#ifdef SGI_RT_TSTAMP_EOB_MODE
 	{ SGI_RT_TSTAMP_EOB_MODE,"SGI_RT_TSTAMP_EOB_MODE"},
 #endif
-#ifdef SGI_USE_FP_BCOPY	
+#ifdef SGI_USE_FP_BCOPY
 	{ SGI_USE_FP_BCOPY,	"SGI_USE_FP_BCOPY"	},
 #endif
-#ifdef SGI_GET_UST		
+#ifdef SGI_GET_UST
 	{ SGI_GET_UST,		"SGI_GET_UST"		},
 #endif
-#ifdef SGI_SPECULATIVE_EXEC	
+#ifdef SGI_SPECULATIVE_EXEC
 	{ SGI_SPECULATIVE_EXEC,	"SGI_SPECULATIVE_EXEC"	},
 #endif
-#ifdef SGI_XLV_NEXT_RQST	
+#ifdef SGI_XLV_NEXT_RQST
 	{ SGI_XLV_NEXT_RQST,	"SGI_XLV_NEXT_RQST"	},
 #endif
-#ifdef SGI_XLV_ATTR_CURSOR	
+#ifdef SGI_XLV_ATTR_CURSOR
 	{ SGI_XLV_ATTR_CURSOR,	"SGI_XLV_ATTR_CURSOR"	},
 #endif
-#ifdef SGI_XLV_ATTR_GET	
+#ifdef SGI_XLV_ATTR_GET
 	{ SGI_XLV_ATTR_GET,	"SGI_XLV_ATTR_GET"	},
 #endif
-#ifdef SGI_XLV_ATTR_SET	
+#ifdef SGI_XLV_ATTR_SET
 	{ SGI_XLV_ATTR_SET,	"SGI_XLV_ATTR_SET"	},
 #endif
 #ifdef SGI_BTOOLSIZE
 	{ SGI_BTOOLSIZE,	"SGI_BTOOLSIZE"		},
 #endif
-#ifdef SGI_BTOOLGET		
+#ifdef SGI_BTOOLGET
 	{ SGI_BTOOLGET,		"SGI_BTOOLGET"		},
 #endif
-#ifdef SGI_BTOOLREINIT		
+#ifdef SGI_BTOOLREINIT
 	{ SGI_BTOOLREINIT,	"SGI_BTOOLREINIT"	},
 #endif
-#ifdef SGI_CREATE_UUID		
+#ifdef SGI_CREATE_UUID
 	{ SGI_CREATE_UUID,	"SGI_CREATE_UUID"	},
 #endif
-#ifdef SGI_NOFPE		
+#ifdef SGI_NOFPE
 	{ SGI_NOFPE,		"SGI_NOFPE"		},
 #endif
-#ifdef SGI_OLD_SOFTFP		
+#ifdef SGI_OLD_SOFTFP
 	{ SGI_OLD_SOFTFP,	"SGI_OLD_SOFTFP"	},
 #endif
-#ifdef SGI_FS_INUMBERS		
+#ifdef SGI_FS_INUMBERS
 	{ SGI_FS_INUMBERS,	"SGI_FS_INUMBERS"	},
 #endif
-#ifdef SGI_FS_BULKSTAT		
+#ifdef SGI_FS_BULKSTAT
 	{ SGI_FS_BULKSTAT,	"SGI_FS_BULKSTAT"	},
 #endif
-#ifdef SGI_RT_TSTAMP_WAIT	
+#ifdef SGI_RT_TSTAMP_WAIT
 	{ SGI_RT_TSTAMP_WAIT,	"SGI_RT_TSTAMP_WAIT"	},
 #endif
-#ifdef SGI_RT_TSTAMP_UPDATE    
+#ifdef SGI_RT_TSTAMP_UPDATE
 	{ SGI_RT_TSTAMP_UPDATE,	"SGI_RT_TSTAMP_UPDATE"	},
 #endif
-#ifdef SGI_PATH_TO_HANDLE	
+#ifdef SGI_PATH_TO_HANDLE
 	{ SGI_PATH_TO_HANDLE,	"SGI_PATH_TO_HANDLE"	},
 #endif
-#ifdef SGI_PATH_TO_FSHANDLE	
+#ifdef SGI_PATH_TO_FSHANDLE
 	{ SGI_PATH_TO_FSHANDLE,	"SGI_PATH_TO_FSHANDLE"	},
 #endif
-#ifdef SGI_FD_TO_HANDLE	
+#ifdef SGI_FD_TO_HANDLE
 	{ SGI_FD_TO_HANDLE,	"SGI_FD_TO_HANDLE"	},
 #endif
-#ifdef SGI_OPEN_BY_HANDLE	
+#ifdef SGI_OPEN_BY_HANDLE
 	{ SGI_OPEN_BY_HANDLE,	"SGI_OPEN_BY_HANDLE"	},
 #endif
-#ifdef SGI_READLINK_BY_HANDLE	
+#ifdef SGI_READLINK_BY_HANDLE
 	{ SGI_READLINK_BY_HANDLE,"SGI_READLINK_BY_HANDLE"},
 #endif
-#ifdef SGI_READ_DANGID		
+#ifdef SGI_READ_DANGID
 	{ SGI_READ_DANGID,	"SGI_READ_DANGID"	},
 #endif
-#ifdef SGI_CONST		
+#ifdef SGI_CONST
 	{ SGI_CONST,		"SGI_CONST"		},
 #endif
-#ifdef SGI_XFS_FSOPERATIONS	
+#ifdef SGI_XFS_FSOPERATIONS
 	{ SGI_XFS_FSOPERATIONS,	"SGI_XFS_FSOPERATIONS"	},
 #endif
-#ifdef SGI_SETASH		
+#ifdef SGI_SETASH
 	{ SGI_SETASH,		"SGI_SETASH"		},
 #endif
-#ifdef SGI_GETASH		
+#ifdef SGI_GETASH
 	{ SGI_GETASH,		"SGI_GETASH"		},
 #endif
-#ifdef SGI_SETPRID		
+#ifdef SGI_SETPRID
 	{ SGI_SETPRID,		"SGI_SETPRID"		},
 #endif
-#ifdef SGI_GETPRID		
+#ifdef SGI_GETPRID
 	{ SGI_GETPRID,		"SGI_GETPRID"		},
 #endif
-#ifdef SGI_SETSPINFO		
+#ifdef SGI_SETSPINFO
 	{ SGI_SETSPINFO,	"SGI_SETSPINFO"		},
 #endif
-#ifdef SGI_GETSPINFO		
+#ifdef SGI_GETSPINFO
 	{ SGI_GETSPINFO,	"SGI_GETSPINFO"		},
 #endif
-#ifdef SGI_SHAREII		
+#ifdef SGI_SHAREII
 	{ SGI_SHAREII,		"SGI_SHAREII"		},
 #endif
-#ifdef SGI_NEWARRAYSESS	
+#ifdef SGI_NEWARRAYSESS
 	{ SGI_NEWARRAYSESS,	"SGI_NEWARRAYSESS"	},
 #endif
-#ifdef SGI_GETDFLTPRID		
+#ifdef SGI_GETDFLTPRID
 	{ SGI_GETDFLTPRID,	"SGI_GETDFLTPRID"	},
 #endif
-#ifdef SGI_SET_DISMISSED_EXC_CNT 
+#ifdef SGI_SET_DISMISSED_EXC_CNT
 	{ SGI_SET_DISMISSED_EXC_CNT,"SGI_SET_DISMISSED_EXC_CNT"	},
 #endif
-#ifdef SGI_GET_DISMISSED_EXC_CNT 
+#ifdef SGI_GET_DISMISSED_EXC_CNT
 	{ SGI_GET_DISMISSED_EXC_CNT,"SGI_GET_DISMISSED_EXC_CNT"	},
 #endif
-#ifdef SGI_CYCLECNTR_SIZE	
+#ifdef SGI_CYCLECNTR_SIZE
 	{ SGI_CYCLECNTR_SIZE,	"SGI_CYCLECNTR_SIZE"	},
 #endif
-#ifdef SGI_QUERY_FASTTIMER	
+#ifdef SGI_QUERY_FASTTIMER
 	{ SGI_QUERY_FASTTIMER,	"SGI_QUERY_FASTTIMER"	},
 #endif
-#ifdef SGI_PIDSINASH		
+#ifdef SGI_PIDSINASH
 	{ SGI_PIDSINASH,	"SGI_PIDSINASH"		},
 #endif
-#ifdef SGI_ULI			
+#ifdef SGI_ULI
 	{ SGI_ULI,		"SGI_ULI"		},
 #endif
-#ifdef SGI_LPG_SHMGET          
+#ifdef SGI_LPG_SHMGET
 	{ SGI_LPG_SHMGET,	"SGI_LPG_SHMGET"	},
 #endif
-#ifdef SGI_LPG_MAP             
+#ifdef SGI_LPG_MAP
 	{ SGI_LPG_MAP,		"SGI_LPG_MAP"		},
 #endif
-#ifdef SGI_CACHEFS_SYS		
+#ifdef SGI_CACHEFS_SYS
 	{ SGI_CACHEFS_SYS,	"SGI_CACHEFS_SYS"	},
 #endif
-#ifdef SGI_NFSNOTIFY		
+#ifdef SGI_NFSNOTIFY
 	{ SGI_NFSNOTIFY,	"SGI_NFSNOTIFY"		},
 #endif
-#ifdef SGI_LOCKDSYS		
+#ifdef SGI_LOCKDSYS
 	{ SGI_LOCKDSYS,		"SGI_LOCKDSYS"		},
 #endif
-#ifdef SGI_EVENTCTR            
+#ifdef SGI_EVENTCTR
 	{ SGI_EVENTCTR,		"SGI_EVENTCTR"		},
 #endif
-#ifdef SGI_GETPRUSAGE          
+#ifdef SGI_GETPRUSAGE
 	{ SGI_GETPRUSAGE,	"SGI_GETPRUSAGE"	},
 #endif
-#ifdef SGI_PROCMASK_LOCATION	
+#ifdef SGI_PROCMASK_LOCATION
 	{ SGI_PROCMASK_LOCATION,"SGI_PROCMASK_LOCATION"	},
 #endif
-#ifdef SGI_UNUSED		
+#ifdef SGI_UNUSED
 	{ SGI_UNUSED,		"SGI_UNUSED"		},
 #endif
-#ifdef SGI_CKPT_SYS		
+#ifdef SGI_CKPT_SYS
 	{ SGI_CKPT_SYS,		"SGI_CKPT_SYS"		},
 #endif
-#ifdef SGI_CKPT_SYS		
+#ifdef SGI_CKPT_SYS
 	{ SGI_CKPT_SYS,		"SGI_CKPT_SYS"		},
 #endif
-#ifdef SGI_GETGRPPID		
+#ifdef SGI_GETGRPPID
 	{ SGI_GETGRPPID,	"SGI_GETGRPPID"		},
 #endif
-#ifdef SGI_GETSESPID		
+#ifdef SGI_GETSESPID
 	{ SGI_GETSESPID,	"SGI_GETSESPID"		},
 #endif
-#ifdef SGI_ENUMASHS		
+#ifdef SGI_ENUMASHS
 	{ SGI_ENUMASHS,		"SGI_ENUMASHS"		},
 #endif
-#ifdef SGI_SETASMACHID		
+#ifdef SGI_SETASMACHID
 	{ SGI_SETASMACHID,	"SGI_SETASMACHID"	},
 #endif
-#ifdef SGI_GETASMACHID		
+#ifdef SGI_GETASMACHID
 	{ SGI_GETASMACHID,	"SGI_GETASMACHID"	},
 #endif
-#ifdef SGI_GETARSESS		
+#ifdef SGI_GETARSESS
 	{ SGI_GETARSESS,	"SGI_GETARSESS"		},
 #endif
-#ifdef SGI_JOINARRAYSESS	
+#ifdef SGI_JOINARRAYSESS
 	{ SGI_JOINARRAYSESS,	"SGI_JOINARRAYSESS"	},
 #endif
-#ifdef SGI_SPROC_KILL		
+#ifdef SGI_SPROC_KILL
 	{ SGI_SPROC_KILL,	"SGI_SPROC_KILL"	},
 #endif
-#ifdef SGI_DBA_CONFIG		
+#ifdef SGI_DBA_CONFIG
 	{ SGI_DBA_CONFIG,	"SGI_DBA_CONFIG"	},
 #endif
-#ifdef SGI_RELEASE_NAME	
+#ifdef SGI_RELEASE_NAME
 	{ SGI_RELEASE_NAME,	"SGI_RELEASE_NAME"	},
 #endif
-#ifdef SGI_SYNCH_CACHE_HANDLER 
+#ifdef SGI_SYNCH_CACHE_HANDLER
 	{ SGI_SYNCH_CACHE_HANDLER,"SGI_SYNCH_CACHE_HANDLER"},
 #endif
-#ifdef SGI_SWASH_INIT		
+#ifdef SGI_SWASH_INIT
 	{ SGI_SWASH_INIT,	"SGI_SWASH_INIT"	},
 #endif
-#ifdef SGI_NUMA_MIGR_PAGE	
+#ifdef SGI_NUMA_MIGR_PAGE
 	{ SGI_NUMA_MIGR_PAGE,	"SGI_NUMA_MIGR_PAGE"	},
 #endif
-#ifdef SGI_NUMA_MIGR_PAGE_ALT	
+#ifdef SGI_NUMA_MIGR_PAGE_ALT
 	{ SGI_NUMA_MIGR_PAGE_ALT,"SGI_NUMA_MIGR_PAGE_ALT"},
 #endif
-#ifdef SGI_KAIO_USERINIT	
+#ifdef SGI_KAIO_USERINIT
 	{ SGI_KAIO_USERINIT,	"SGI_KAIO_USERINIT"	},
 #endif
-#ifdef SGI_KAIO_READ		
+#ifdef SGI_KAIO_READ
 	{ SGI_KAIO_READ,	"SGI_KAIO_READ"		},
 #endif
-#ifdef SGI_KAIO_WRITE		
+#ifdef SGI_KAIO_WRITE
 	{ SGI_KAIO_WRITE,	"SGI_KAIO_WRITE"	},
 #endif
-#ifdef SGI_KAIO_SUSPEND	
+#ifdef SGI_KAIO_SUSPEND
 	{ SGI_KAIO_SUSPEND,	"SGI_KAIO_SUSPEND"	},
 #endif
-#ifdef SGI_KAIO_STATS		
+#ifdef SGI_KAIO_STATS
 	{ SGI_KAIO_STATS,	"SGI_KAIO_STATS"	},
 #endif
-#ifdef SGI_INITIAL_PT_SPROC	
+#ifdef SGI_INITIAL_PT_SPROC
 	{ SGI_INITIAL_PT_SPROC,	"SGI_INITIAL_PT_SPROC"	},
 #endif
 	{ 0,			NULL			},
@@ -1156,7 +1141,7 @@ struct uio;
 #include <sys/fs/nfs.h>
 #include <sys/fs/nfs_clnt.h>
 
-static struct xlat mount_flags[] = {
+static const struct xlat mount_flags[] = {
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_FSS,	"MS_FSS"	},
 	{ MS_DATA,	"MS_DATA"	},
@@ -1170,7 +1155,7 @@ static struct xlat mount_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat nfs_flags[] = {
+static const struct xlat nfs_flags[] = {
 	{ NFSMNT_SOFT,		"NFSMNT_SOFT"		},
 	{ NFSMNT_WSIZE,		"NFSMNT_WSIZE"		},
 	{ NFSMNT_RSIZE,		"NFSMNT_RSIZE"		},
@@ -1257,7 +1242,7 @@ struct tcb *tcp;
 
 #include <sys/fs/vx_ioctl.h>
 
-static struct xlat mount_flags[] = {
+static const struct xlat mount_flags[] = {
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_FSS,	"MS_FSS"	},
 	{ MS_DATA,	"MS_DATA"	},
@@ -1271,7 +1256,7 @@ static struct xlat mount_flags[] = {
 };
 
 #ifdef VX_MS_MASK
-static struct xlat vxfs_flags[] = {
+static const struct xlat vxfs_flags[] = {
 	{ VX_MS_NOLOG,		"VX_MS_NOLOG"		},
 	{ VX_MS_BLKCLEAR,	"VX_MS_BLKCLEAR"	},
 	{ VX_MS_SNAPSHOT,	"VX_MS_SNAPSHOT"	},
@@ -1293,7 +1278,7 @@ static struct xlat vxfs_flags[] = {
 };
 #endif
 
-static struct xlat nfs_flags[] = {
+static const struct xlat nfs_flags[] = {
 	{ NFSMNT_SOFT,		"NFSMNT_SOFT"		},
 	{ NFSMNT_WSIZE,		"NFSMNT_WSIZE"		},
 	{ NFSMNT_RSIZE,		"NFSMNT_RSIZE"		},
@@ -1353,7 +1338,7 @@ struct tcb *tcp;
 					if (args.mflags & VX_MS_SNAPSHOT) {
 						tprintf (", snapof=");
 						printstr (tcp,
-							  (long) args.primaryspec, 
+							  (long) args.primaryspec,
 							  -1);
 						if (args.snapsize > 0)
 							tprintf (", snapsize=%ld", args.snapsize);
@@ -1419,7 +1404,7 @@ struct tcb *tcp;
 
 #ifdef SYS_capget
 
-static struct xlat capabilities[] = {
+static const struct xlat capabilities[] = {
 	{ 1<<CAP_CHOWN,		"CAP_CHOWN"	},
 	{ 1<<CAP_DAC_OVERRIDE,	"CAP_DAC_OVERRIDE"},
 	{ 1<<CAP_DAC_READ_SEARCH,"CAP_DAC_READ_SEARCH"},
@@ -1498,7 +1483,7 @@ struct tcb *tcp;
 			tprintf(", ");
 			printflags(capabilities, arg1->inheritable);
 			tprintf("}");
-		} 
+		}
 	}
 	return 0;
 }
@@ -1550,7 +1535,7 @@ struct tcb *tcp;
 			tprintf(", ");
 			printflags(capabilities, arg1->inheritable);
 			tprintf("}");
-		} 
+		}
 	}
 	return 0;
 }
@@ -1572,7 +1557,7 @@ struct tcb *tcp;
 #endif
 
 #ifdef LINUX
-static struct xlat sysctl_root[] = {
+static const struct xlat sysctl_root[] = {
 	{ CTL_KERN, "CTL_KERN" },
 	{ CTL_VM, "CTL_VM" },
 	{ CTL_NET, "CTL_NET" },
@@ -1583,7 +1568,7 @@ static struct xlat sysctl_root[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_kern[] = {
+static const struct xlat sysctl_kern[] = {
 	{ KERN_OSTYPE, "KERN_OSTYPE" },
 	{ KERN_OSRELEASE, "KERN_OSRELEASE" },
 	{ KERN_OSREV, "KERN_OSREV" },
@@ -1625,21 +1610,61 @@ static struct xlat sysctl_kern[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_vm[] = {
+static const struct xlat sysctl_vm[] = {
+#ifdef VM_SWAPCTL
 	{ VM_SWAPCTL, "VM_SWAPCTL" },
+#endif
+#ifdef VM_UNUSED1
+	{ VM_UNUSED1, "VM_UNUSED1" },
+#endif
+#ifdef VM_SWAPOUT
 	{ VM_SWAPOUT, "VM_SWAPOUT" },
+#endif
+#ifdef VM_UNUSED2
+	{ VM_UNUSED2, "VM_UNUSED2" },
+#endif
+#ifdef VM_FREEPG
 	{ VM_FREEPG, "VM_FREEPG" },
+#endif
+#ifdef VM_UNUSED3
+	{ VM_UNUSED3, "VM_UNUSED3" },
+#endif
+#ifdef VM_BDFLUSH
 	{ VM_BDFLUSH, "VM_BDFLUSH" },
+#endif
+#ifdef VM_UNUSED4
+	{ VM_UNUSED4, "VM_UNUSED4" },
+#endif
 	{ VM_OVERCOMMIT_MEMORY, "VM_OVERCOMMIT_MEMORY" },
+#ifdef VM_BUFFERMEM
 	{ VM_BUFFERMEM, "VM_BUFFERMEM" },
+#endif
+#ifdef VM_UNUSED5
+	{ VM_UNUSED5, "VM_UNUSED5" },
+#endif
+#ifdef VM_PAGECACHE
 	{ VM_PAGECACHE, "VM_PAGECACHE" },
+#endif
+#ifdef VM_UNUSED7
+	{ VM_UNUSED7, "VM_UNUSED7" },
+#endif
+#ifdef VM_PAGERDAEMON
 	{ VM_PAGERDAEMON, "VM_PAGERDAEMON" },
+#endif
+#ifdef VM_UNUSED8
+	{ VM_UNUSED8, "VM_UNUSED8" },
+#endif
+#ifdef VM_PGT_CACHE
 	{ VM_PGT_CACHE, "VM_PGT_CACHE" },
+#endif
+#ifdef VM_UNUSED9
+	{ VM_UNUSED9, "VM_UNUSED9" },
+#endif
 	{ VM_PAGE_CLUSTER, "VM_PAGE_CLUSTER" },
 	{ 0, NULL },
 };
 
-static struct xlat sysctl_net[] = {
+static const struct xlat sysctl_net[] = {
 	{ NET_CORE, "NET_CORE" },
 	{ NET_ETHER, "NET_ETHER" },
 	{ NET_802, "NET_802" },
@@ -1658,7 +1683,7 @@ static struct xlat sysctl_net[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_core[] = {
+static const struct xlat sysctl_net_core[] = {
 	{ NET_CORE_WMEM_MAX, "NET_CORE_WMEM_MAX" },
 	{ NET_CORE_RMEM_MAX, "NET_CORE_RMEM_MAX" },
 	{ NET_CORE_WMEM_DEFAULT, "NET_CORE_WMEM_DEFAULT" },
@@ -1671,13 +1696,13 @@ static struct xlat sysctl_net_core[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_unix[] = {
+static const struct xlat sysctl_net_unix[] = {
 	{ NET_UNIX_DESTROY_DELAY, "NET_UNIX_DESTROY_DELAY" },
 	{ NET_UNIX_DELETE_DELAY, "NET_UNIX_DELETE_DELAY" },
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv4[] = {
+static const struct xlat sysctl_net_ipv4[] = {
 	{ NET_IPV4_FORWARD, "NET_IPV4_FORWARD" },
 	{ NET_IPV4_DYNADDR, "NET_IPV4_DYNADDR" },
 	{ NET_IPV4_CONF, "NET_IPV4_CONF" },
@@ -1720,7 +1745,7 @@ static struct xlat sysctl_net_ipv4[] = {
 	{  0, NULL }
 };
 
-static struct xlat sysctl_net_ipv4_route[] = {
+static const struct xlat sysctl_net_ipv4_route[] = {
 	{ NET_IPV4_ROUTE_FLUSH, "NET_IPV4_ROUTE_FLUSH" },
 	{ NET_IPV4_ROUTE_MIN_DELAY, "NET_IPV4_ROUTE_MIN_DELAY" },
 	{ NET_IPV4_ROUTE_MAX_DELAY, "NET_IPV4_ROUTE_MAX_DELAY" },
@@ -1738,7 +1763,7 @@ static struct xlat sysctl_net_ipv4_route[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv4_conf[] = {
+static const struct xlat sysctl_net_ipv4_conf[] = {
 	{ NET_IPV4_CONF_FORWARDING, "NET_IPV4_CONF_FORWARDING" },
 	{ NET_IPV4_CONF_MC_FORWARDING, "NET_IPV4_CONF_MC_FORWARDING" },
 	{ NET_IPV4_CONF_PROXY_ARP, "NET_IPV4_CONF_PROXY_ARP" },
@@ -1753,14 +1778,14 @@ static struct xlat sysctl_net_ipv4_conf[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv6[] = {
+static const struct xlat sysctl_net_ipv6[] = {
 	{ NET_IPV6_CONF, "NET_IPV6_CONF" },
 	{ NET_IPV6_NEIGH, "NET_IPV6_NEIGH" },
 	{ NET_IPV6_ROUTE, "NET_IPV6_ROUTE" },
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv6_route[] = {
+static const struct xlat sysctl_net_ipv6_route[] = {
 	{ NET_IPV6_ROUTE_FLUSH, "NET_IPV6_ROUTE_FLUSH" },
 	{ NET_IPV6_ROUTE_GC_THRESH, "NET_IPV6_ROUTE_GC_THRESH" },
 	{ NET_IPV6_ROUTE_MAX_SIZE, "NET_IPV6_ROUTE_MAX_SIZE" },
@@ -1925,7 +1950,7 @@ struct tcb *tcp;
 	char ctl[1024];
 	size_t len;
 	int i, numeric;
-	
+
 	if (entering(tcp)) {
 		if (tcp->u_arg[1] < 0 || tcp->u_arg[1] > CTL_MAXNAME ||
 		    (umoven(tcp, tcp->u_arg[0], tcp->u_arg[1] * sizeof(int),
@@ -1956,7 +1981,7 @@ struct tcb *tcp;
 		if (!syserror(tcp) && (umove(tcp, tcp->u_arg[3], &len) >= 0)) {
 			printstr(tcp, tcp->u_arg[2], len);
 			tprintf(", [%u], ", len);
-		} else 
+		} else
 			tprintf("%#lx, %#lx, ", tcp->u_arg[2], tcp->u_arg[3]);
 		printstr(tcp, tcp->u_arg[4], tcp->u_arg[5]);
 		tprintf(", %lu", tcp->u_arg[5]);
@@ -1970,10 +1995,10 @@ struct tcb *tcp;
 #include <sys/ksym.h>
 #include <sys/elf.h>
 
-static struct xlat ksym_flags[] = {
-	{ STT_NOTYPE,	"STT_NOTYPE"	},	
-	{ STT_FUNC,	"STT_FUNC"	},	
-	{ STT_OBJECT,	"STT_OBJECT"	},	
+static const struct xlat ksym_flags[] = {
+	{ STT_NOTYPE,	"STT_NOTYPE"	},
+	{ STT_FUNC,	"STT_FUNC"	},
+	{ STT_OBJECT,	"STT_OBJECT"	},
 	{ 0,		NULL		},
 };
 
@@ -2010,9 +2035,10 @@ struct tcb *tcp;
 
 #ifdef HAVE_SYS_NSCSYS_H
 
+struct cred;
 #include <sys/nscsys.h>
 
-static struct xlat ssi_cmd [] = {
+static const struct xlat ssi_cmd [] = {
 	{ SSISYS_BADOP,	"SSISYS_BADOP"	},
 	{ SSISYS_LDLVL_INIT,"SSISYS_LDLVL_INIT"},
 	{ SSISYS_LDLVL_GETVEC,"SSISYS_LDLVL_GETVEC"},
@@ -2063,8 +2089,11 @@ int sys_ssisys (tcp)
 struct tcb *tcp;
 {
 	struct ssisys_iovec iov;
-	
+	cls_nodeinfo_args_t cni;
+	clusternode_info_t info;
+
 	if (entering (tcp)) {
+		ts_reclaim_child_inargs_t trc;
 		if (tcp->u_arg[1] != sizeof iov ||
 		    umove (tcp, tcp->u_arg[0], &iov) < 0)
 		{
@@ -2075,7 +2104,22 @@ struct tcb *tcp;
 		printxval(ssi_cmd, iov.tio_id.id_cmd, "SSISYS_???");
 		tprintf (":%d", iov.tio_id.id_ver);
 		switch (iov.tio_id.id_cmd) {
+		    case SSISYS_RECLAIM_CHILD:
+			if (iov.tio_udatainlen != sizeof trc ||
+			    umove (tcp, (long) iov.tio_udatain, &trc) < 0)
+				goto bad;
+			tprintf (", in={pid=%ld, start=%ld}",
+				 trc.trc_pid, trc.trc_start);
+			break;
+		    case SSISYS_CLUSTERNODE_INFO:
+			if (iov.tio_udatainlen != sizeof cni ||
+			    umove (tcp, (long) iov.tio_udatain, &cni) < 0)
+				goto bad;
+			tprintf (", in={node=%ld, len=%d}",
+				 cni.nodenum, cni.info_len);
+			break;
 		    default:
+		    bad:
 			if (iov.tio_udatainlen) {
 				tprintf (", in=[/* %d bytes */]",
 					 iov.tio_udatainlen);
@@ -2083,13 +2127,31 @@ struct tcb *tcp;
 		}
 	}
 	else {
+		if (tcp->u_arg[1] != sizeof iov ||
+		    umove (tcp, tcp->u_arg[0], &iov) < 0)
+		    goto done;
 		switch (iov.tio_id.id_cmd) {
+		    case SSISYS_CLUSTERNODE_INFO:
+			if (iov.tio_udatainlen != sizeof cni ||
+			    umove (tcp, (long) iov.tio_udatain, &cni) < 0)
+				goto bad_out;
+			if (cni.info_len != sizeof info ||
+			    iov.tio_udataoutlen != sizeof &info ||
+			    umove (tcp, (long) iov.tio_udataout, &info) < 0)
+				goto bad_out;
+			tprintf (", out={node=%ld, cpus=%d, online=%d}",
+				 info.node_num, info.node_totalcpus,
+				 info.node_onlinecpus);
+			break;
+
 		    default:
+		    bad_out:
 			if (iov.tio_udataoutlen) {
 				tprintf (", out=[/* %d bytes */]",
 					 iov.tio_udataoutlen);
 			}
 		}
+	    done:
 		tprintf ("}, %ld", tcp->u_arg[1]);
 	}
 	return 0;
@@ -2105,7 +2167,7 @@ struct tcb *tcp;
 #define __NEW_UTS_LEN 64
 #endif
 
-static struct xlat sysmips_operations[] = {
+static const struct xlat sysmips_operations[] = {
 	{ SETNAME,		"SETNAME"	},
 	{ FLUSH_CACHE,		"FLUSH_CACHE"	},
 	{ MIPS_FIXADE,		"MIPS_FIXADE"	},
