@@ -24,22 +24,31 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: term.c,v 1.2 1999/05/09 00:29:59 wichert Exp $
+ *	$Id: term.c,v 1.7 2004/09/04 03:39:20 roland Exp $
  */
 
 #include "defs.h"
 
+#ifdef LINUX
+/*
+ * The C library's definition of struct termios might differ from
+ * the kernel one, and we need to use the kernel layout.
+ */
+#include <linux/termios.h>
+#else
+
 #ifdef HAVE_TERMIO_H
 #include <termio.h>
 #endif /* HAVE_TERMIO_H */
 
 #include <termios.h>
+#endif
 
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>
 #endif
 
-static struct xlat tcxonc_options[] = {
+static const struct xlat tcxonc_options[] = {
 	{ TCOOFF,	"TCOOFF"	},
 	{ TCOON,	"TCOON"		},
 	{ TCIOFF,	"TCIOFF"	},
@@ -47,14 +56,16 @@ static struct xlat tcxonc_options[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat tcflsh_options[] = {
+#ifdef TCLFLSH
+static const struct xlat tcflsh_options[] = {
 	{ TCIFLUSH,	"TCIFLUSH"	},
 	{ TCOFLUSH,	"TCOFLUSH"	},
 	{ TCIOFLUSH,	"TCIOFLUSH"	},
 	{ 0,		NULL		},
 };
+#endif
 
-static struct xlat baud_options[] = {
+static const struct xlat baud_options[] = {
 	{ B0,		"B0"		},
 	{ B50,		"B50"		},
 	{ B75,		"B75"		},
@@ -75,6 +86,51 @@ static struct xlat baud_options[] = {
 #ifdef B38400
 	{ B38400,	"B38400"	},
 #endif
+#ifdef B57600
+	{ B57600,	"B57600"	},
+#endif
+#ifdef B115200
+	{ B115200,	"B115200"	},
+#endif
+#ifdef B230400
+	{ B230400,	"B230400"	},
+#endif
+#ifdef B460800
+	{ B460800,	"B460800"	},
+#endif
+#ifdef B500000
+	{ B500000,	"B500000"	},
+#endif
+#ifdef B576000
+	{ B576000,	"B576000"	},
+#endif
+#ifdef B921600
+	{ B921600,	"B921600"	},
+#endif
+#ifdef B1000000
+	{ B1000000,	"B1000000"	},
+#endif
+#ifdef B1152000
+	{ B1152000,	"B1152000"	},
+#endif
+#ifdef B1500000
+	{ B1500000,	"B1500000"	},
+#endif
+#ifdef B2000000
+	{ B2000000,	"B2000000"	},
+#endif
+#ifdef B2500000
+	{ B2500000,	"B2500000"	},
+#endif
+#ifdef B3000000
+	{ B3000000,	"B3000000"	},
+#endif
+#ifdef B3500000
+	{ B3500000,	"B3500000"	},
+#endif
+#ifdef B4000000
+	{ B4000000,	"B4000000"	},
+#endif
 #ifdef EXTA
 	{ EXTA,		"EXTA"		},
 #endif
@@ -84,7 +140,7 @@ static struct xlat baud_options[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat modem_flags[] = {
+static const struct xlat modem_flags[] = {
 #ifdef TIOCM_LE
 	{ TIOCM_LE,	"TIOCM_LE",	},
 #endif
@@ -128,7 +184,14 @@ struct tcb *tcp;
 long code, arg;
 {
 	struct termios tios;
+#ifndef FREEBSD
 	struct termio tio;
+#else
+	#define TCGETS	TIOCGETA
+	#define TCSETS	TIOCSETA
+	#define TCSETSW	TIOCSETAW
+	#define TCSETSF	TIOCSETAF
+#endif
 	struct winsize ws;
 #ifdef TIOCGSIZE
 	struct  ttysize ts;
@@ -153,7 +216,16 @@ long code, arg;
 			return 0;
 		if (abbrev(tcp)) {
 			tprintf(", {");
+#ifndef FREEBSD
 			printxval(baud_options, tios.c_cflag & CBAUD, "B???");
+#else
+			printxval(baud_options, tios.c_ispeed, "B???");
+			if (tios.c_ispeed != tios.c_ospeed) {
+				tprintf(" (in)");
+				printxval(baud_options, tios.c_ospeed, "B???");
+				tprintf(" (out)");
+			}
+#endif
 			tprintf(" %sopost %sisig %sicanon %secho ...}",
 				(tios.c_oflag & OPOST) ? "" : "-",
 				(tios.c_lflag & ISIG) ? "" : "-",
@@ -165,7 +237,7 @@ long code, arg;
 			(long) tios.c_iflag, (long) tios.c_oflag);
 		tprintf("c_cflags=%#lx, c_lflags=%#lx, ",
 			(long) tios.c_cflag, (long) tios.c_lflag);
-#ifndef SVR4
+#if !defined(SVR4) && !defined(FREEBSD)
 		tprintf("c_line=%u, ", tios.c_line);
 #endif
 		if (!(tios.c_lflag & ICANON))
@@ -245,15 +317,18 @@ long code, arg;
 #endif
 
 	/* ioctls with a direct decodable arg */
-
+#ifdef TCXONC
 	case TCXONC:
 		tprintf(", ");
 		printxval(tcxonc_options, arg, "TC???");
 		return 1;
+#endif
+#ifdef TCLFLSH
 	case TCFLSH:
 		tprintf(", ");
 		printxval(tcflsh_options, arg, "TC???");
 		return 1;
+#endif
 
 	/* ioctls with an indirect parameter displayed as modem flags */
 
@@ -385,4 +460,3 @@ long code, arg;
 		return 0;
 	}
 }
-
