@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: time.c,v 1.4 2000/10/13 12:47:12 wichert Exp $
+ *	$Id: time.c,v 1.11 2004/10/06 22:27:43 roland Exp $
  */
 
 #include "defs.h"
@@ -34,6 +34,8 @@
 #ifdef LINUX
 #include <linux/version.h>
 #include <sys/timex.h>
+#include <linux/ioctl.h>
+#include <linux/rtc.h>
 #endif /* LINUX */
 
 void
@@ -54,7 +56,7 @@ long addr;
 }
 
 #ifdef ALPHA
-struct timeval32 
+struct timeval32
 {
     unsigned tv_sec;
     unsigned tv_usec;
@@ -188,7 +190,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat which[] = {
+static const struct xlat which[] = {
 	{ ITIMER_REAL,	"ITIMER_REAL"	},
 	{ ITIMER_VIRTUAL,"ITIMER_VIRTUAL"},
 	{ ITIMER_PROF,	"ITIMER_PROF"	},
@@ -358,5 +360,256 @@ struct tcb *tcp;
 	}
 	return 0;
 }
-#endif /* LINUX */
 
+static const struct xlat clockflags[] = {
+  { TIMER_ABSTIME, "TIMER_ABSTIME" },
+  { 0,             NULL }
+};
+
+static const struct xlat clocknames[] = {
+#ifdef CLOCK_REALTIME
+  { CLOCK_REALTIME, "CLOCK_REALTIME" },
+#endif
+#ifdef CLOCK_MONOTONIC
+  { CLOCK_MONOTONIC, "CLOCK_MONOTONIC" },
+#endif
+  { 0,             NULL }
+};
+
+int
+sys_clock_settime(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printxval(clocknames, tcp->u_arg[0], "CLOCK_???");
+		tprintf(", ");
+		printtv(tcp, tcp->u_arg[1]);
+	}
+	return 0;
+}
+
+int
+sys_clock_gettime(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printxval(clocknames, tcp->u_arg[0], "CLOCK_???");
+		tprintf(", ");
+	} else {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[1]);
+		else
+			printtv(tcp, tcp->u_arg[1]);
+	}
+	return 0;
+}
+
+int
+sys_clock_nanosleep(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printxval(clocknames, tcp->u_arg[0], "CLOCK_???");
+		tprintf(", ");
+		if (printflags(clockflags, tcp->u_arg[1]) == 0)
+			tprintf("0");
+		tprintf(", ");
+		printtv(tcp, tcp->u_arg[2]);
+		tprintf(", ");
+	} else {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[3]);
+		else
+			printtv(tcp, tcp->u_arg[3]);
+	}
+	return 0;
+}
+
+#ifndef SIGEV_THREAD_ID
+# define SIGEV_THREAD_ID 4
+#endif
+static const struct xlat sigev_value[] = {
+	{ SIGEV_SIGNAL+1, "SIGEV_SIGNAL" },
+	{ SIGEV_NONE+1, "SIGEV_NONE" },
+	{ SIGEV_THREAD+1, "SIGEV_THREAD" },
+	{ SIGEV_THREAD_ID+1, "SIGEV_THREAD_ID" },
+	{ 0, NULL }
+};
+
+void
+printsigevent(tcp, arg)
+struct tcb *tcp;
+long arg;
+{
+	struct sigevent sev;
+	if (umove (tcp, arg, &sev) < 0)
+		tprintf("{...}");
+	else {
+		tprintf("{%p, ", sev.sigev_value.sival_ptr);
+		if (sev.sigev_notify == SIGEV_SIGNAL)
+			tprintf("%s, ", signame(sev.sigev_signo));
+		else
+			tprintf("%u, ", sev.sigev_signo);
+		printxval(sigev_value, sev.sigev_notify+1, "SIGEV_???");
+		tprintf(", ");
+		if (sev.sigev_notify == SIGEV_THREAD_ID)
+			/* _pad[0] is the _tid field which might not be
+			   present in the userlevel definition of the
+			   struct.  */
+			tprintf("{%d}", sev._sigev_un._pad[0]);
+		else if (sev.sigev_notify == SIGEV_THREAD)
+			tprintf("{%p, %p}", sev.sigev_notify_function,
+				sev.sigev_notify_attributes);
+		else
+			tprintf("{...}");
+		tprintf("}");
+	}
+}
+
+int
+sys_timer_create(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printxval(clocknames, tcp->u_arg[0], "CLOCK_???");
+		tprintf(", ");
+		printsigevent(tcp, tcp->u_arg[1]);
+		tprintf(", ");
+	} else {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[2]);
+		else {
+			void *p;
+			umove(tcp, tcp->u_arg[2], &p);
+			tprintf("{%p}", p);
+		}
+	}
+	return 0;
+}
+
+int
+sys_timer_settime(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%#lx, ", tcp->u_arg[0]);
+		if (printflags(clockflags, tcp->u_arg[1]) == 0)
+			tprintf("0");
+		tprintf(", ");
+		printitv(tcp, tcp->u_arg[2]);
+		tprintf(", ");
+	} else {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[3]);
+		else
+			printitv(tcp, tcp->u_arg[3]);
+	}
+	return 0;
+}
+
+int
+sys_timer_gettime(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%#lx, ", tcp->u_arg[0]);
+	} else {
+		if (syserror(tcp))
+			tprintf("%#lx", tcp->u_arg[1]);
+		else
+			printitv(tcp, tcp->u_arg[1]);
+	}
+	return 0;
+}
+
+static void
+print_rtc(tcp, rt)
+struct tcb *tcp;
+const struct rtc_time *rt;
+{
+	tprintf("{tm_sec=%d, tm_min=%d, tm_hour=%d, "
+		"tm_mday=%d, tm_mon=%d, tm_year=%d, ",
+		rt->tm_sec, rt->tm_min, rt->tm_hour,
+		rt->tm_mday, rt->tm_mon, rt->tm_year);
+	if (!abbrev(tcp))
+		tprintf("tm_wday=%d, tm_yday=%d, tm_isdst=%d}",
+			rt->tm_wday, rt->tm_yday, rt->tm_isdst);
+	else
+		tprintf("...}");
+}
+
+int
+rtc_ioctl(tcp, code, arg)
+struct tcb *tcp;
+long code;
+long arg;
+{
+	switch (code) {
+	case RTC_ALM_SET:
+	case RTC_SET_TIME:
+		if (entering(tcp)) {
+			struct rtc_time rt;
+			if (umove(tcp, arg, &rt) < 0)
+				tprintf(", %#lx", arg);
+			else {
+				tprintf(", ");
+				print_rtc(tcp, &rt);
+			}
+		}
+		break;
+	case RTC_ALM_READ:
+	case RTC_RD_TIME:
+		if (exiting(tcp)) {
+			struct rtc_time rt;
+			if (syserror(tcp) || umove(tcp, arg, &rt) < 0)
+				tprintf(", %#lx", arg);
+			else {
+				tprintf(", ");
+				print_rtc(tcp, &rt);
+			}
+		}
+		break;
+	case RTC_IRQP_SET:
+	case RTC_EPOCH_SET:
+		if (entering(tcp))
+			tprintf(", %lu", arg);
+		break;
+	case RTC_IRQP_READ:
+	case RTC_EPOCH_READ:
+		if (exiting(tcp))
+			tprintf(", %lu", arg);
+		break;
+	case RTC_WKALM_SET:
+		if (entering(tcp)) {
+			struct rtc_wkalrm wk;
+			if (umove(tcp, arg, &wk) < 0)
+				tprintf(", %#lx", arg);
+			else {
+				tprintf(", {enabled=%d, pending=%d, ",
+					wk.enabled, wk.pending);
+				print_rtc(tcp, &wk.time);
+				tprintf("}");
+			}
+		}
+		break;
+	case RTC_WKALM_RD:
+		if (exiting(tcp)) {
+			struct rtc_wkalrm wk;
+			if (syserror(tcp) || umove(tcp, arg, &wk) < 0)
+				tprintf(", %#lx", arg);
+			else {
+				tprintf(", {enabled=%d, pending=%d, ",
+					wk.enabled, wk.pending);
+				print_rtc(tcp, &wk.time);
+				tprintf("}");
+			}
+		}
+		break;
+	default:
+		if (entering(tcp))
+			tprintf(", %#lx", arg);
+		break;
+	}
+	return 1;
+}
+#endif /* LINUX */
